@@ -22,6 +22,7 @@
 */
 
 #include <config.h>
+#include <libgnome/gnome-i18n.h>
 #include "control-data.h"
 #include "spell.h"
 #include "properties.h"
@@ -37,13 +38,18 @@ gtk_html_control_data_new (GtkHTML *html, GtkWidget *vbox)
 	ncd->properties_dialog       = NULL;
 	ncd->properties_types        = NULL;
 	ncd->block_font_style_change = FALSE;
-	ncd->dict_client             = spell_new_dictionary ();
-	ncd->dict                    = ncd->dict_client ? bonobo_object_corba_objref (BONOBO_OBJECT (ncd->dict_client)) : NULL;
+	ncd->dict                    = spell_new_dictionary ();
 	ncd->gdk_painter             = NULL;
 	ncd->plain_painter           = NULL;
 	ncd->format_html             = FALSE;
 	ncd->control                 = NULL;
 
+	ncd->search_text             = NULL;
+	ncd->replace_text_search     = NULL;
+	ncd->replace_text_replace    = NULL;
+	ncd->has_spell_control_set   = FALSE;
+	ncd->language                = NULL;
+
 	spell_init (html, ncd);
 
 	return ncd;
@@ -63,16 +69,17 @@ gtk_html_control_data_destroy (GtkHTMLControlData *cd)
 	if (cd->replace_dialog)
 		gtk_html_replace_dialog_destroy (cd->replace_dialog);
 
-	if (cd->dict_client)
-		bonobo_object_unref (BONOBO_OBJECT (cd->dict_client));
+	/* printf ("release dict\n"); */
+	bonobo_object_release_unref (cd->dict, NULL);
 
 	if (cd->plain_painter) 
-		gtk_object_unref (GTK_OBJECT (cd->plain_painter));
+		g_object_unref (cd->plain_painter);
 
 	if (cd->gdk_painter) 
-		gtk_object_unref (GTK_OBJECT (cd->gdk_painter));
+		g_object_unref (cd->gdk_painter);
+
+	if (cd->languages)
+		CORBA_free (cd->languages);
 
 	g_free (cd);
 }
-
-
