@@ -27,6 +27,7 @@
 #include <config.h>
 #endif
 
+#include <libgnome/gnome-i18n.h>
 #include <string.h>
 #include <gnome.h>
 #include <bonobo.h>
@@ -41,10 +42,6 @@
 
 #include <glade/glade.h>
 
-#ifdef GNOME_GTKHTML_EDITOR_SHLIB
-#include <bonobo/bonobo-shlib-factory.h>
-#endif
-
 #include "Editor.h"
 
 #include "gtkhtml.h"
@@ -97,12 +94,32 @@ typedef struct _SetFrameData SetFrameData;
 
 static GtkHTMLEditorAPI *editor_api;
 
+static void
+activate_ui_cb (BonoboControl      *control,
+	     gboolean            active,
+	     GtkHTMLControlData *cd)
+{
+	BonoboUIComponent *ui_component;
+
+	ui_component = bonobo_control_get_ui_component (control);
+
+	if (active) {
+		Bonobo_UIContainer remote_ui_container;
+
+		cd->uic = ui_component;
+
+		remote_ui_container = bonobo_control_get_remote_ui_container (control, NULL);
+		bonobo_ui_component_set_container (ui_component, remote_ui_container, NULL);
+		bonobo_object_release_unref (remote_ui_container, NULL);
+
+		menubar_setup (ui_component, cd);
+	}
+}
+
 static void
 set_frame_cb (BonoboControl *control,
 	      gpointer data)
 {
-	Bonobo_UIContainer remote_ui_container;
-	BonoboUIComponent *ui_component;
 	GtkHTMLControlData *control_data;
 	GtkWidget *toolbar;
 	GtkWidget *scrolled_window;
@@ -113,11 +130,10 @@ set_frame_cb (BonoboControl *control,
 	frame = bonobo_control_get_control_frame (control, NULL);
 	if (frame == CORBA_OBJECT_NIL)
 		return;
+
 	CORBA_Object_release (frame, NULL);
 
-	remote_ui_container = bonobo_control_get_remote_ui_container (control, NULL);
-	control_data->uic = ui_component = bonobo_control_get_ui_component (control);
-	bonobo_ui_component_set_container (ui_component, remote_ui_container, NULL);
+	control_data->uic = bonobo_control_get_ui_component (control);
 
 	/* Setup the tool bar.  */
 
@@ -132,19 +148,10 @@ set_frame_cb (BonoboControl *control,
 
 	gtk_box_pack_start (GTK_BOX (control_data->vbox), scrolled_window, TRUE, TRUE, 0);
 
-	/* Setup the menu bar.  */
-
-	menubar_setup (ui_component, control_data);
-
-	if (!spell_has_control ()) {
-		control_data->has_spell_control = FALSE;
-		bonobo_ui_component_set_prop (ui_component, "/commands/EditSpellCheck", "sensitive", "0", NULL);
-	} else
-		control_data->has_spell_control = TRUE;
+	/* hack to setup the bonobo ui if activate isn't working */
+	activate_ui_cb (control, TRUE, control_data);
 
 	gtk_html_set_editor_api (GTK_HTML (control_data->html), editor_api, control_data);
-
-	bonobo_object_release_unref (remote_ui_container, NULL);
 }
 
 static gint
@@ -233,12 +240,17 @@ load_from_file (GtkHTML *html,
 	unsigned char buffer[4096];
 	int len;
 	int fd;
-        const char *path;
+        char *path;
 
         if (strncmp (url, "file:", 5) == 0)
-		path = url + 5; 
+		path = g_filename_from_uri(url, NULL, NULL);
+	else
+		path = g_strdup(url);
 
-	if ((fd = open (path, O_RDONLY)) == -1) {
+	fd = open (path, O_RDONLY);
+	g_free(path);
+
+	if (fd == -1) {
 		g_warning ("%s", g_strerror (errno));
 		return FALSE;
 	}
@@ -289,16 +301,21 @@ url_requested_cb (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpointe
 	}
 }
 
+static gboolean
+html_show_popup(GtkWidget *html, GtkHTMLControlData *cd)
+{
+	popup_show_at_cursor (cd);
+
+	return TRUE;
+}
+
 static gint
 html_button_pressed (GtkWidget *html, GdkEventButton *event, GtkHTMLControlData *cd)
 {
 	HTMLEngine *engine = cd->html->engine;
 	guint offset;
 
-	cd->obj = html_engine_get_object_at (engine,
-					     event->x + engine->x_offset,
-					     event->y + engine->y_offset,
-					     &offset, FALSE);
+	cd->obj = html_engine_get_object_at (engine, event->x, event->y, &offset, FALSE);
 	switch (event->button) {
 	case 1:
 		if (event->type == GDK_2BUTTON_PRESS && cd->obj && event->state & GDK_CONTROL_MASK) {
@@ -313,9 +330,7 @@ html_button_pressed (GtkWidget *html, GdkEventButton *event, GtkHTMLControlData
 	case 3:
 		if (!html_engine_is_selection_active (engine) || !html_engine_point_in_selection (engine, cd->obj, offset)) {
 			html_engine_disable_selection (engine);
-			html_engine_jump_at (engine,
-					     event->x + engine->x_offset,
-					     event->y + engine->y_offset);
+			html_engine_jump_at (engine, event->x, event->y);
 			gtk_html_update_styles (cd->html);
 		}
 
@@ -358,26 +373,22 @@ html_button_pressed_after (GtkWidget *html, GdkEventButton *event, GtkHTMLContro
 static void
 editor_init_painters (GtkHTMLControlData *cd)
 {	
-	GtkHTMLClassProperties *prop;
 	GtkHTML *html;
 
 	g_return_if_fail (cd != NULL);
 
+
 	html = cd->html;
-	prop = GTK_HTML_CLASS (GTK_OBJECT_GET_CLASS (html))->properties;
+	
+	gtk_widget_ensure_style (GTK_WIDGET (html));
 	
 	if (!cd->plain_painter) {
+		cd->gdk_painter = HTML_GDK_PAINTER (html->engine->painter);
 		cd->plain_painter = HTML_GDK_PAINTER (html_plain_painter_new (GTK_WIDGET (html), TRUE));
-		html_font_manager_set_default (&HTML_PAINTER (cd->plain_painter)->font_manager,
-					       prop->font_var,      prop->font_fix,
-					       prop->font_var_size, prop->font_var_points,
-					       prop->font_fix_size, prop->font_fix_points);
-		
+
 		html_colorset_add_slave (html->engine->settings->color_set, 
 					 HTML_PAINTER (cd->plain_painter)->color_set);
 
-		cd->gdk_painter = HTML_GDK_PAINTER (html->engine->painter);
-
 		/* the plain painter starts with a ref */
 		g_object_ref (G_OBJECT (cd->gdk_painter));
 	}	
@@ -404,7 +415,7 @@ editor_set_format (GtkHTMLControlData *cd, gboolean format_html)
 		old_p = cd->gdk_painter;
 	}		
 
-	printf ("set format %d\n", format_html);
+	/* printf ("set format %d\n", format_html); */
 
 	toolbar_update_format (cd);
 	menubar_update_format (cd);
@@ -414,15 +425,26 @@ editor_set_format (GtkHTMLControlData *cd, gboolean format_html)
 		if (html->engine->window)
 			html_gdk_painter_realize (p, html->engine->window);
 
+		html_font_manager_set_default (&HTML_PAINTER (p)->font_manager,
+					       HTML_PAINTER (old_p)->font_manager.variable.face,
+					       HTML_PAINTER (old_p)->font_manager.fixed.face,
+					       HTML_PAINTER (old_p)->font_manager.var_size,
+					       HTML_PAINTER (old_p)->font_manager.var_points,
+					       HTML_PAINTER (old_p)->font_manager.fix_size,
+					       HTML_PAINTER (old_p)->font_manager.fix_points);
+
 		html_engine_set_painter (html->engine, HTML_PAINTER (p));
 		html_engine_schedule_redraw (html->engine);
 	}
 		
 }
 
-enum {
+static enum {
 	PROP_EDIT_HTML,
-	PROP_HTML_TITLE
+	PROP_HTML_TITLE,
+	PROP_INLINE_SPELLING,
+	PROP_MAGIC_LINKS,
+	PROP_MAGIC_SMILEYS
 } EditorControlProps;
 
 static void
@@ -441,6 +463,15 @@ editor_get_prop (BonoboPropertyBag *bag,
 	case PROP_HTML_TITLE:
 		BONOBO_ARG_SET_STRING (arg, gtk_html_get_title (cd->html));
 		break;
+	case PROP_INLINE_SPELLING:
+		BONOBO_ARG_SET_BOOLEAN (arg, gtk_html_get_inline_spelling (cd->html));
+		break;
+	case PROP_MAGIC_LINKS:
+		BONOBO_ARG_SET_BOOLEAN (arg, gtk_html_get_magic_links (cd->html));
+		break;
+	case PROP_MAGIC_SMILEYS:
+		BONOBO_ARG_SET_BOOLEAN (arg, gtk_html_get_magic_smileys (cd->html));
+		break;
        	default:
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
 		break;
@@ -464,6 +495,15 @@ editor_set_prop (BonoboPropertyBag *bag,
 	case PROP_HTML_TITLE:
 		gtk_html_set_title (cd->html, BONOBO_ARG_GET_STRING (arg));
 		break;
+	case PROP_INLINE_SPELLING:
+		gtk_html_set_inline_spelling (cd->html, BONOBO_ARG_GET_BOOLEAN (arg));
+		break;
+	case PROP_MAGIC_LINKS:
+		gtk_html_set_magic_links (cd->html, BONOBO_ARG_GET_BOOLEAN (arg));
+		break;
+	case PROP_MAGIC_SMILEYS:
+		gtk_html_set_magic_smileys (cd->html, BONOBO_ARG_GET_BOOLEAN (arg));
+		break;
 	default:
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
 		break;
@@ -488,6 +528,7 @@ editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 	html_widget = gtk_html_new ();
 	gtk_html_load_empty (GTK_HTML (html_widget));
 	gtk_html_set_editable (GTK_HTML (html_widget), TRUE);
+	gtk_html_set_animate (GTK_HTML (html_widget), FALSE);
 
 	cd = gtk_html_control_data_new (GTK_HTML (html_widget), vbox);
 	g_signal_connect (control, "destroy", G_CALLBACK (control_destroy), cd);
@@ -518,6 +559,36 @@ editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 
 	CORBA_free (def);
 
+	def = bonobo_arg_new (BONOBO_ARG_BOOLEAN);
+	BONOBO_ARG_SET_BOOLEAN (def, gtk_html_get_inline_spelling (GTK_HTML (html_widget)));
+
+	bonobo_property_bag_add (pb, "InlineSpelling", PROP_INLINE_SPELLING,
+				 BONOBO_ARG_BOOLEAN, def,
+				 "Include spelling errors inline", 
+				 0);
+
+	CORBA_free (def);
+
+	def = bonobo_arg_new (BONOBO_ARG_BOOLEAN);
+	BONOBO_ARG_SET_BOOLEAN (def, gtk_html_get_magic_links (GTK_HTML (html_widget)));
+
+	bonobo_property_bag_add (pb, "MagicLinks", PROP_MAGIC_LINKS,
+				 BONOBO_ARG_BOOLEAN, def,
+				 "Recognize links in text and replace them", 
+				 0);
+
+	CORBA_free (def);
+
+	def = bonobo_arg_new (BONOBO_ARG_BOOLEAN);
+	BONOBO_ARG_SET_BOOLEAN (def, gtk_html_get_magic_smileys (GTK_HTML (html_widget)));
+
+	bonobo_property_bag_add (pb, "MagicSmileys", PROP_MAGIC_SMILEYS,
+				 BONOBO_ARG_BOOLEAN, def,
+				 "Recognize smileys in text and replace them", 
+				 0);
+
+	CORBA_free (def);
+
 	def = bonobo_arg_new (BONOBO_ARG_STRING);
 	BONOBO_ARG_SET_STRING (def, "");
 
@@ -526,10 +597,20 @@ editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 				 "The title of the html document", 
 				 0);
 	CORBA_free (def);
+
 	/*
 	def = bonobo_arg_new (BONOBO_ARG_STRING);
 	BONOBO_ARG_SET_STRING (def, "");
 
+	bonobo_property_bag_add (pb, "SpellLanguage", PROP_CURRENT_LANGUAGE,
+				 BONOBO_ARG_STRING, def,
+				 "The title of the html document", 
+				 0);
+	CORBA_free (def);
+	
+	def = bonobo_arg_new (BONOBO_ARG_STRING);
+	BONOBO_ARG_SET_STRING (def, "");
+
 	bonobo_property_bag_add (pb, "OnURL", PROP_EDIT_HTML,
 				 BONOBO_ARG_STRING, def,
 				 "The URL of the link the mouse is currently over", 
@@ -539,14 +620,18 @@ editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 	bonobo_control_set_properties (control, BONOBO_OBJREF (pb), NULL);
 	bonobo_object_unref (BONOBO_OBJECT (pb));
 
+
+
 	/* Part of the initialization must be done after the control is
 	   embedded in its control frame.  We use the "set_frame" signal to
 	   handle that.  */
 
+	/* g_signal_connect (control, "activate", G_CALLBACK (activate_ui_cb), cd); */
 	g_signal_connect (control, "set_frame", G_CALLBACK (set_frame_cb), cd);
 	g_signal_connect (html_widget, "url_requested", G_CALLBACK (url_requested_cb), cd);
 	g_signal_connect (html_widget, "button_press_event", G_CALLBACK (html_button_pressed), cd);
 	g_signal_connect_after (html_widget, "button_press_event", G_CALLBACK (html_button_pressed_after), cd);
+	g_signal_connect (html_widget, "popup_menu", G_CALLBACK(html_show_popup), cd);
 
 	cd->control = control;
 }
@@ -587,7 +672,7 @@ send_event_str (GNOME_GtkHTML_Editor_Engine engine, GNOME_GtkHTML_Editor_Listene
 	/* printf ("sending to listener\n"); */
 	CORBA_exception_init (&ev);
 	bonobo_retval = GNOME_GtkHTML_Editor_Listener_event (listener, name, bonobo_arg, &ev);
-	CORBA_free (bonobo_arg);
+	bonobo_arg_release (bonobo_arg);
 
 	if (ev._major == CORBA_NO_EXCEPTION) {
 		if (!bonobo_arg_type_is_equal (bonobo_retval->_type, TC_null, &ev)
@@ -608,11 +693,15 @@ send_event_void (GNOME_GtkHTML_Editor_Engine engine, GNOME_GtkHTML_Editor_Listen
 {
 	CORBA_Environment ev;
 	CORBA_any *any;
+	CORBA_any *retval;
 
 	any = CORBA_any_alloc ();
 	any->_type = TC_null;
 	CORBA_exception_init (&ev);
-	GNOME_GtkHTML_Editor_Listener_event (listener, name, any, &ev);
+	retval = GNOME_GtkHTML_Editor_Listener_event (listener, name, any, &ev);
+	if (!BONOBO_EX (&ev))
+	    CORBA_free (retval);
+
 	CORBA_exception_free (&ev);
 	CORBA_free (any);
 }
@@ -625,6 +714,7 @@ send_event_stream (GNOME_GtkHTML_Editor_Engine engine,
 		   GtkHTMLStream *stream) 
 {
 	CORBA_any *any;
+	CORBA_any *retval;
 	GNOME_GtkHTML_Editor_URLRequestEvent e;
 	CORBA_Environment ev;
 	BonoboObject *bstream;
@@ -638,7 +728,9 @@ send_event_stream (GNOME_GtkHTML_Editor_Engine engine,
 	e.stream = BONOBO_OBJREF (bstream);
 	
 	CORBA_exception_init (&ev);
-	GNOME_GtkHTML_Editor_Listener_event (listener, name, any, &ev);
+	retval = GNOME_GtkHTML_Editor_Listener_event (listener, name, any, &ev);
+	if (!BONOBO_EX (&ev))
+	    CORBA_free (retval);
 
 	bonobo_object_unref (BONOBO_OBJECT (bstream));
 	CORBA_exception_free (&ev);
@@ -727,13 +819,13 @@ editor_control_init (void)
 	}
 }
 
-static BonoboObject *
+BonoboObject *
 editor_control_factory (BonoboGenericFactory *factory, const gchar *component_id, gpointer closure)
 {
 	BonoboControl *control;
 	GtkWidget *vbox;
 
-	printf ("factory: %s\n", component_id);
+	/* printf ("factory: %s\n", component_id); */
 
 	editor_control_init ();
 
@@ -742,6 +834,7 @@ editor_control_factory (BonoboGenericFactory *factory, const gchar *component_id
 
 	/* g_warning ("Creating a new GtkHTML editor control."); */
 	control = bonobo_control_new (vbox);
+	/* printf ("%s\n", bonobo_ui_component_get_name (bonobo_control_get_ui_component (control))); */
 
 	if (control){
 		editor_control_construct (control, vbox);
@@ -751,49 +844,3 @@ editor_control_factory (BonoboGenericFactory *factory, const gchar *component_id
 		return NULL;
 	}
 }
-
-#ifdef GNOME_GTKHTML_EDITOR_SHLIB
-
-BONOBO_OAF_SHLIB_FACTORY (CONTROL_FACTORY_ID, "GNOME HTML Editor factory", editor_control_factory, NULL);
-
-#else
-
-int
-main (int argc, char **argv)
-{
-	BonoboGenericFactory *factory;
-#ifdef GTKHTML_HAVE_GCONF
-	GError  *gconf_error  = NULL;
-#endif
-
-	/* Initialize the i18n support */
-	bindtextdomain(GTKHTML_RELEASE_STRING, GNOMELOCALEDIR);
-	textdomain(GTKHTML_RELEASE_STRING);
-
-	gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv, 
-			   GNOME_PROGRAM_STANDARD_PROPERTIES,
-			   GNOME_PARAM_HUMAN_READABLE_NAME, _("GtkHTML Editor Control"),			   
-			   NULL);
-
-	/* #ifdef GTKHTML_HAVE_GCONF
-	if (!gconf_init (argc, argv, &gconf_error)) {
-		g_assert (gconf_error != NULL);
-		g_error ("GConf init failed:\n  %s", gconf_error->message);
-		return 1;
-	}
-	#endif */
-
-	factory = bonobo_generic_factory_new (CONTROL_FACTORY_ID, editor_control_factory, NULL);
-
-	if (factory) {
-		bonobo_running_context_auto_exit_unref (BONOBO_OBJECT (factory));
-	
-		bonobo_activate ();
-		bonobo_main ();
-
-		return bonobo_ui_debug_shutdown ();
-	} else
-		return 1;
-
-}
-#endif /* GNOME_GTKHTML_EDITOR_SHLIB */
