@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  This file is part of the GtkHTML library.
 
-    Copyright (C) 2000 Helix Code, Inc.
+    Copyright (C) 2002 Ximian, Inc.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -18,13 +18,18 @@
     the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
 
-    Author: Ettore Perazzoli <ettore@helixcode.com>
+    Author: Radek Doulik
+
 */
 
-#ifndef _EDITOR_CONTROL_FACTORY_H
-#define _EDITOR_CONTROL_FACTORY_H
+#ifndef EDITOR_CONTROL_FACTORY_H_
+#define EDITOR_CONTROL_FACTORY_H_
 
-
-void editor_control_factory_init (void);
+#include <bonobo.h>
 
-#endif /* _EDITOR_CONTROL_FACTORY_H */
+#define CONTROL_FACTORY_ID "OAFIID:GNOME_GtkHTML_Editor_Factory:" EDITOR_API_VERSION
+#define CONTROL_ID         "OAFIID:GNOME_GtkHTML_Editor:" EDITOR_API_VERSION
+
+BonoboObject *editor_control_factory (BonoboGenericFactory *factory, const gchar *component_id, gpointer closure);
+
+#endif
