@@ -21,11 +21,10 @@
 */
 
 #include <config.h>
+#include <libgnome/gnome-i18n.h>
 #include <unistd.h>
 #include <string.h>
 #include <glade/glade.h>
-#include <gal/widgets/e-unicode.h>
-#include <gal/util/e-unicode-i18n.h>
 
 #include "gtkhtml.h"
 #include "htmlcolorset.h"
@@ -190,7 +189,7 @@ get_location (GtkHTMLEditImageProperties *d)
 	if (file) {
 		url = g_strconcat ("file://", file, NULL);
 	} else {
-		GtkWidget *entry = gnome_pixmap_entry_gtk_entry (GNOME_PIXMAP_ENTRY (d->pentry));
+		GtkWidget *entry = gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (d->pentry));
 
 		url = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
 	}
@@ -254,12 +253,22 @@ get_sample_html (GtkHTMLEditImageProperties *d, gboolean insert)
 	if (insert) {
 		html   = g_strconcat (body, image, NULL);
 	} else {
-		html   = g_strconcat (body,
-				      U_("The quick brown fox jumped over the lazy dog."),
+		if (strcasecmp(location, "") != 0) {
+			html = g_strconcat (body,
+				      _("This is sample text, designed to show "
+					"you how text would flow around this "
+					"image if you insert the image into "
+					"your message."),
 				      " ",
 				      image,
-				      U_("The quick brown fox jumped over the lazy dog."),
+				      _("This is sample text, designed to show "
+					"you how text would flow around this "
+					"image if you insert the image into "
+					"your message."),
 				      NULL);
+		} else {
+			html = g_strconcat (body, _("No image selected"), NULL);
+		}
 	}
 
 	g_free (location);
@@ -291,7 +300,7 @@ fill_sample (GtkHTMLEditImageProperties *d)
 static void
 pentry_changed (GtkWidget *entry, GtkHTMLEditImageProperties *d)
 {
-	gchar *text;
+	const gchar *text;
 
 	text = gtk_entry_get_text (GTK_ENTRY (entry));
 	if (!text || !d->location || strcmp (text, d->location)) {
@@ -310,7 +319,7 @@ static void
 url_changed (GtkWidget *entry, GtkHTMLEditImageProperties *d)
 {
 	g_free (d->url);
-	d->url = e_utf8_from_gtk_string (entry, gtk_entry_get_text (GTK_ENTRY (entry)));
+	d->url = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
 	CHANGE;
 	FILL;
 }
@@ -319,7 +328,7 @@ static void
 alt_changed (GtkWidget *entry, GtkHTMLEditImageProperties *d)
 {
 	g_free (d->alt);
-	d->alt = e_utf8_from_gtk_string (entry, gtk_entry_get_text (GTK_ENTRY (entry)));
+	d->alt = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
 	CHANGE;
 	FILL;
 }
@@ -356,7 +365,7 @@ test_url_clicked (GtkWidget *w, GtkHTMLEditImageProperties *d)
 	const char *url = gtk_entry_get_text (GTK_ENTRY (d->entry_url));
 
 	if (url)
-		gnome_url_show (url);
+		gnome_url_show (url, NULL);
 }
 
 static void
@@ -368,7 +377,7 @@ fill_templates (GtkHTMLEditImageProperties *d)
 	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_template));
 
 	for (i = 0; i < TEMPLATES; i ++)
-		gtk_menu_append (GTK_MENU (menu), gtk_menu_item_new_with_label (_(image_templates [i].name)));
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), gtk_menu_item_new_with_label (_(image_templates [i].name)));
 	gtk_menu_set_active (GTK_MENU (menu), 0);
 	gtk_container_remove (GTK_CONTAINER (menu), gtk_menu_get_active (GTK_MENU (menu)));
 }
@@ -376,9 +385,6 @@ fill_templates (GtkHTMLEditImageProperties *d)
 static void
 set_ui (GtkHTMLEditImageProperties *d)
 {
-	gchar *url;
-	gchar *alt;
-
 	d->disable_change = TRUE;
 
 	gtk_option_menu_set_history (GTK_OPTION_MENU (d->option_template), d->template);
@@ -391,16 +397,9 @@ set_ui (GtkHTMLEditImageProperties *d)
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (d->spin_padh), d->padh);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (d->spin_padv), d->padv);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (d->spin_border), d->border);
-
-	url = e_utf8_to_gtk_string (d->entry_url, d->url ? d->url : "");
-	gtk_entry_set_text (GTK_ENTRY (d->entry_url), url);
-	g_free (url);
-
-	alt = e_utf8_to_gtk_string (d->entry_alt, d->alt);
-	gtk_entry_set_text (GTK_ENTRY (d->entry_alt), alt ? alt : "");
-	g_free (alt);
-
-	gtk_entry_set_text (GTK_ENTRY (gnome_pixmap_entry_gtk_entry (GNOME_PIXMAP_ENTRY (d->pentry))),
+	gtk_entry_set_text (GTK_ENTRY (d->entry_url), d->url ? d->url : "");
+	gtk_entry_set_text (GTK_ENTRY (d->entry_alt), d->alt ? d->alt : "");
+	gtk_entry_set_text (GTK_ENTRY (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (d->pentry))),
 			    d->location ? d->location : "");
 
 	gtk_widget_set_sensitive (d->spin_width, d->width_percent != 2);
@@ -414,8 +413,12 @@ set_ui (GtkHTMLEditImageProperties *d)
 static void
 changed_template (GtkWidget *w, GtkHTMLEditImageProperties *d)
 {
+	gint oldtemplate = d->template;
 	d->template = g_list_index (GTK_MENU_SHELL (w)->children, gtk_menu_get_active (GTK_MENU (w)));
 
+	if (d->template == oldtemplate)
+		return;
+
 	d->border = image_templates [d->template].border;
 	d->align = image_templates [d->template].align;
 	d->padh = image_templates [d->template].padh;
@@ -537,7 +540,7 @@ image_widget (GtkHTMLEditImageProperties *d, gboolean insert)
 	GladeXML *xml;
 	GtkWidget *frame_template;
 
-	xml = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "image_page");
+	xml = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "image_page", GETTEXT_PACKAGE);
 	if (!xml)
 		g_error (_("Could not load glade file."));
 
@@ -546,53 +549,50 @@ image_widget (GtkHTMLEditImageProperties *d, gboolean insert)
 	frame_template = glade_xml_get_widget (xml, "frame_image_template");
 
 	d->option_align = glade_xml_get_widget (xml, "option_image_align");
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_align))),
-			    "selection-done", changed_align, d);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_align)),
+			  "selection-done", G_CALLBACK (changed_align), d);
 	d->option_width_percent = glade_xml_get_widget (xml, "option_image_width_percent");
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_width_percent))),
-			    "selection-done", changed_width_percent, d);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_width_percent)),
+			  "selection-done", G_CALLBACK (changed_width_percent), d);
 	d->option_height_percent = glade_xml_get_widget (xml, "option_image_height_percent");
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_height_percent))),
-			    "selection-done", changed_height_percent, d);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_height_percent)),
+			  "selection-done", G_CALLBACK (changed_height_percent), d);
 
 	d->spin_border = glade_xml_get_widget (xml, "spin_image_border");
 	UPPER_FIX (border);
-	gtk_signal_connect (GTK_OBJECT (d->spin_border), "changed", changed_border, d);
+	g_signal_connect (d->spin_border, "value_changed", G_CALLBACK (changed_border), d);
 	d->spin_width = glade_xml_get_widget (xml, "spin_image_width");
 	UPPER_FIX (width);
-	gtk_signal_connect (GTK_OBJECT (d->spin_width), "changed", changed_width, d);
+	g_signal_connect (d->spin_width, "value_changed", G_CALLBACK (changed_width), d);
 	d->spin_height = glade_xml_get_widget (xml, "spin_image_height");
 	UPPER_FIX (height);
-	gtk_signal_connect (GTK_OBJECT (d->spin_height), "changed", changed_height, d);
+	g_signal_connect (d->spin_height, "value_changed", G_CALLBACK (changed_height), d);
 	d->spin_padh = glade_xml_get_widget (xml, "spin_image_padh");
 	UPPER_FIX (padh);
-	gtk_signal_connect (GTK_OBJECT (d->spin_padh), "changed", changed_padh, d);
+	g_signal_connect (d->spin_padh, "value_changed", G_CALLBACK (changed_padh), d);
 	d->spin_padv = glade_xml_get_widget (xml, "spin_image_padv");
 	UPPER_FIX (padv);
-	gtk_signal_connect (GTK_OBJECT (d->spin_padv), "changed", changed_padv, d);
+	g_signal_connect (d->spin_padv, "value_changed", G_CALLBACK (changed_padv), d);
 
 	d->option_template = glade_xml_get_widget (xml, "option_image_template");
-	gtk_signal_connect (GTK_OBJECT (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_template))),
-			    "selection-done", changed_template, d);
+	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_template)),
+			  "selection-done", G_CALLBACK (changed_template), d);
 	if (insert)
 		fill_templates (d);
 
 	gtk_container_add (GTK_CONTAINER (d->frame_sample), sample_frame (&d->sample));
-	gtk_signal_disconnect_by_func (GTK_OBJECT (d->sample), url_requested, NULL);
-	gtk_signal_connect (GTK_OBJECT (d->sample), "url_requested", image_url_requested, d);
+	g_signal_handlers_disconnect_matched (d->sample, G_SIGNAL_MATCH_FUNC, 0, 0, NULL, G_CALLBACK (url_requested), NULL);
+	g_signal_connect (GTK_OBJECT (d->sample), "url_requested", G_CALLBACK (image_url_requested), d);
 
 	d->entry_url = glade_xml_get_widget (xml, "entry_image_url");
-	gtk_signal_connect (GTK_OBJECT (d->entry_url), "changed", GTK_SIGNAL_FUNC (url_changed), d);
+	g_signal_connect (GTK_OBJECT (d->entry_url), "changed", G_CALLBACK (url_changed), d);
 
 	d->entry_alt = glade_xml_get_widget (xml, "entry_image_alt");
-	gtk_signal_connect (GTK_OBJECT (d->entry_alt), "changed", GTK_SIGNAL_FUNC (alt_changed), d);
+	g_signal_connect (d->entry_alt, "changed", G_CALLBACK (alt_changed), d);
 
 	d->pentry = glade_xml_get_widget (xml, "pentry_image_location");
-	gtk_signal_connect (GTK_OBJECT (gnome_pixmap_entry_gtk_entry (GNOME_PIXMAP_ENTRY (d->pentry))),
-			    "changed", GTK_SIGNAL_FUNC (pentry_changed), d);
-	/* fix for broken gnome-libs, could be removed once gnome-libs are fixed */
-	gnome_entry_load_history (GNOME_ENTRY (gnome_pixmap_entry_gnome_entry (GNOME_PIXMAP_ENTRY (d->pentry))));
-	our_gnome_pixmap_entry_set_last_pixmap (GNOME_PIXMAP_ENTRY (d->pentry));
+	g_signal_connect (GTK_OBJECT (gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (d->pentry))),
+			    "changed", G_CALLBACK (pentry_changed), d);
 
 	gtk_widget_show_all (d->page);
 	if (!insert)
@@ -676,7 +676,7 @@ image_properties (GtkHTMLControlData *cd, gpointer *set_data)
 	return w;
 }
 
-static void
+static gboolean
 insert_or_apply (GtkHTMLControlData *cd, gpointer get_data, gboolean insert)
 {	
 	GtkHTMLEditImageProperties *d = (GtkHTMLEditImageProperties *) get_data;
@@ -688,10 +688,27 @@ insert_or_apply (GtkHTMLControlData *cd, gpointer get_data, gboolean insert)
 		gtk_html_append_html (d->cd->html, html);
 	} else {
 		HTMLImage *image = HTML_IMAGE (d->image);
+		HTMLEngine *e = d->cd->html->engine;
 		gchar *location, *url, *target;
+		gint position;
+
+		position = e->cursor->position;
 
 		g_assert (HTML_OBJECT_TYPE (d->image) == HTML_TYPE_IMAGE);
 
+		if (e->cursor->object != HTML_OBJECT (d->image))
+			if (!html_cursor_jump_to (e->cursor, e, HTML_OBJECT (d->image), 1)) {
+				GtkWidget *dialog;
+				printf ("d: %p\n", d->cd->properties_dialog);
+				dialog = gtk_message_dialog_new (GTK_WINDOW (d->cd->properties_dialog->dialog),
+								 GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+								 _("The editted image was removed from the document.\nCannot apply your changes."));
+				gtk_dialog_run (GTK_DIALOG (dialog));
+				gtk_widget_destroy (dialog);
+				html_cursor_jump_to_position (e->cursor, e, position);
+				return FALSE;
+			}
+
 		html_image_set_border (image, d->border);
 		html_image_set_size (image,
 				     d->width_percent == 2 ? 0 : d->width,
@@ -723,19 +740,23 @@ insert_or_apply (GtkHTMLControlData *cd, gpointer get_data, gboolean insert)
 		if (target)
 			g_free (url);
 		g_free (target);
+		html_cursor_jump_to_position (e->cursor, e, position);
 	}
+
+	return TRUE;
 }
 
-void
+gboolean
 image_apply_cb (GtkHTMLControlData *cd, gpointer get_data)
 {
-	insert_or_apply (cd, get_data, FALSE);
+	return insert_or_apply (cd, get_data, FALSE);
 }
 
-void
+gboolean
 image_insert_cb (GtkHTMLControlData *cd, gpointer get_data)
 {
 	insert_or_apply (cd, get_data, TRUE);
+	return TRUE;
 }
 
 void
