@@ -24,9 +24,10 @@
 #include <config.h>
 #endif
 
-#include <gal/widgets/e-unicode.h>
-
+#include <libgnome/gnome-i18n.h>
+#include <string.h>
 #include "gtkhtml.h"
+#include "gtkhtml-private.h"
 
 #include "htmlclueflow.h"
 #include "htmlcursor.h"
@@ -259,7 +260,7 @@ show_prop_dialog (GtkHTMLControlData *cd, GtkHTMLEditPropertyType start)
 static void
 prop_dialog (GtkWidget *mi, GtkHTMLControlData *cd)
 {
-	show_prop_dialog (cd, GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (mi), "type")));
+	show_prop_dialog (cd, GPOINTER_TO_INT (g_object_get_data (G_OBJECT (mi), "type")));
 }
 
 static void
@@ -283,11 +284,11 @@ link_prop_dialog (GtkWidget *mi, GtkHTMLControlData *cd)
 static void
 spell_suggest (GtkWidget *mi, GtkHTMLControlData *cd)
 {
-	HTMLEngine *e = cd->html->engine;
+	/* HTMLEngine *e = cd->html->engine;
 
-	/* gtk_signal_emit_by_name (GTK_OBJECT (cd->html), "spell_suggestion_request",
+	   gtk_signal_emit_by_name (GTK_OBJECT (cd->html), "spell_suggestion_request",
 	   e->spell_checker, html_engine_get_word (e)); */
-	spell_suggestion_request (cd->html, html_engine_get_spell_word (e), cd);
+	spell_suggestion_request (cd->html, cd);
 }
 
 static void
@@ -304,7 +305,7 @@ spell_add (GtkWidget *mi, GtkHTMLControlData *cd)
 
 	word = html_engine_get_spell_word (e);
 	if (word) {
-		spell_add_to_personal (cd->html, word, cd);
+		spell_add_to_personal (cd->html, word, g_object_get_data (G_OBJECT (mi), "abbrev"), cd);
 		g_free (word);
 	}
 	html_engine_spell_check (e);
@@ -345,39 +346,40 @@ insert_html (GtkWidget *mi, GtkHTMLControlData *cd)
 #endif
 
 #define ADD_ITEM_BASE(f,t) \
-                gtk_object_set_data (GTK_OBJECT (menuitem), "type", GINT_TO_POINTER (GTK_HTML_EDIT_PROPERTY_ ## t)); \
-		gtk_menu_append (GTK_MENU (menu), menuitem); \
+                g_object_set_data (G_OBJECT (menuitem), "type", GINT_TO_POINTER (GTK_HTML_EDIT_PROPERTY_ ## t)); \
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem); \
 		gtk_widget_show (menuitem); \
-		gtk_signal_connect (GTK_OBJECT (menuitem), "activate", GTK_SIGNAL_FUNC (f), cd); \
+		g_signal_connect (menuitem, "activate", G_CALLBACK (f), cd); \
                 (*items)++; items_sep++
 
 #define ADD_ITEM(l,f,t) \
 		menuitem = gtk_menu_item_new_with_label (l); \
                 ADD_ITEM_BASE (f,t)
 
-#define ADD_ITEM_SENSITIVE(l,f,t,s) \
-		menuitem = gtk_menu_item_new_with_label (l); \
-                ADD_ITEM_BASE (f,t); \
-                gtk_widget_set_sensitive (menuitem, s);
+#define ADD_STOCK(i,f) \
+		menuitem = gtk_image_menu_item_new_from_stock (i, NULL); \
+                ADD_ITEM_BASE (f,NONE);
 
-#define ADD_ITEM_UTF8(l,f,t) \
-                menuitem = e_utf8_gtk_menu_item_new_with_label (GTK_MENU (menu), l); \
-                ADD_ITEM_BASE (f,t)
+#define ADD_STOCK_SENSITIVE(i,f,s) \
+		menuitem = gtk_image_menu_item_new_from_stock (i, NULL); \
+                ADD_ITEM_BASE (f,NONE); \
+                gtk_widget_set_sensitive (menuitem, s);
 
 #define ADD_SEP \
         if (items_sep) { \
                 menuitem = gtk_menu_item_new (); \
-                gtk_menu_append (GTK_MENU (menu), menuitem); \
+                gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem); \
                 gtk_widget_show (menuitem); \
 		items_sep = 0; \
         }
 
 #define ADD_PROP(x) \
-        cd->properties_types = g_list_append (cd->properties_types, GINT_TO_POINTER (GTK_HTML_EDIT_PROPERTY_ ## x))
+        cd->properties_types = g_list_append (cd->properties_types, GINT_TO_POINTER (GTK_HTML_EDIT_PROPERTY_ ## x)); \
+        (*props) ++;
 
 #define SUBMENU(l) \
-		        menuitem = gtk_menu_item_new_with_label (_(l)); \
-			gtk_menu_append (GTK_MENU (menu), menuitem); \
+		        menuitem = gtk_menu_item_new_with_label (l); \
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem); \
 			gtk_widget_show (menuitem); \
 			(*items)++; items_sep++; \
 			submenu = gtk_menu_new (); \
@@ -389,8 +391,36 @@ insert_html (GtkWidget *mi, GtkHTMLControlData *cd)
 			gtk_widget_show (menu); \
 			menu = menuparent
 
+static gint
+get_n_languages (GtkHTMLControlData *cd)
+{
+	gint i, n = 0;
+
+	if (cd->languages)
+		for (i = 0; i < cd->languages->_length; i ++)
+			if (strstr (html_engine_get_language (cd->html->engine), cd->languages->_buffer [i].abbreviation))
+				n ++;
+
+	return n;
+}
+
+static const gchar *
+get_language (GtkHTMLControlData *cd)
+{
+	gint i;
+	const gchar *abbrev = NULL;
+
+	if (cd->languages)
+		for (i = 0; i < cd->languages->_length; i ++)
+			if (strstr (html_engine_get_language (cd->html->engine), cd->languages->_buffer [i].abbreviation)) {
+				abbrev = cd->languages->_buffer [i].abbreviation;
+			}
+
+	return abbrev;
+}
+
 static GtkWidget *
-prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
+prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items, guint *props)
 {
 	HTMLEngine *e = cd->html->engine;
 	HTMLObject *obj;
@@ -402,6 +432,8 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
 
 	obj  = cd->html->engine->cursor->object;
 	menu = gtk_menu_new ();
+	*items = 0;
+	*props = 0;
 
 	if (cd->properties_types) {
 		g_list_free (cd->properties_types);
@@ -415,13 +447,13 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
 	ADD_SEP;
 #endif
 	active = html_engine_is_selection_active (e);
-	ADD_ITEM (_("Undo"), undo, NONE); 
-	ADD_ITEM (_("Redo"), redo, NONE); 
+	ADD_STOCK (GTK_STOCK_UNDO, undo); 
+	ADD_STOCK (GTK_STOCK_REDO, redo); 
 
 	ADD_SEP;
-	ADD_ITEM_SENSITIVE (_("Copy"), copy, NONE, active);
-	ADD_ITEM_SENSITIVE (_("Cut"),  cut, NONE, active);
-	ADD_ITEM (_("Paste"),  paste, NONE);
+	ADD_STOCK_SENSITIVE (GTK_STOCK_CUT,  cut, active);
+	ADD_STOCK_SENSITIVE (GTK_STOCK_COPY, copy, active);
+	ADD_STOCK (GTK_STOCK_PASTE, paste);
 	ADD_ITEM (_("Paste Quotation"),  paste_cite, NONE);
 
 	ADD_SEP;
@@ -436,78 +468,53 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
 		ADD_ITEM (_("Remove link"), remove_link, NONE);
 	}
 
-	if (!active && obj && html_object_is_text (obj)
-	    && !html_engine_spell_word_is_valid (e)) {
-		gchar *spell, *word, *check_utf8, *add_utf8, *ignore_utf8, *ignore, *add;
-
-		word   = html_engine_get_spell_word (e);
-		check_utf8 = e_utf8_from_locale_string (_("Check '%s' spelling..."));
-		spell  = g_strdup_printf (check_utf8, word);
-		g_free (check_utf8);
-		add_utf8 = e_utf8_from_locale_string (_("Add '%s' to dictionary"));
-		add    = g_strdup_printf (add_utf8, word);
-		g_free (add_utf8);
-		ignore_utf8 = e_utf8_from_locale_string (_("Ignore '%s'"));
-		ignore = g_strdup_printf (ignore_utf8, word);
-		g_free (ignore_utf8);
-		ADD_SEP;
-		SUBMENU (N_("Spell checker"));
-		if (cd->has_spell_control) {
-			ADD_ITEM_UTF8 (spell, spell_check_cb, NONE);
-		} else {
-			ADD_ITEM (_("Suggest word"), spell_suggest, NONE);
-		}
-		ADD_ITEM_UTF8 (add, spell_add, NONE);
-		ADD_ITEM_UTF8 (ignore, spell_ignore, NONE);
-		END_SUBMENU;
-
-		g_free (spell);
-		g_free (add);
-		g_free (ignore);
-		g_free (word);
-	}
-
 	if (cd->format_html && obj) {
+		ADD_SEP;
+		SUBMENU (_("Style"));
 		switch (HTML_OBJECT_TYPE (obj)) {
 		case HTML_TYPE_TEXT:
-			ADD_SEP;
-			ADD_ITEM (_("Text..."), prop_dialog, TEXT);
+			ADD_ITEM (_("Text Style..."), prop_dialog, TEXT);
 			ADD_PROP (TEXT);
-			ADD_ITEM (_("Paragraph..."), prop_dialog, PARAGRAPH);
+			ADD_ITEM (_("Paragraph Style..."), prop_dialog, PARAGRAPH);
 			ADD_PROP (PARAGRAPH);
 			break;
 		case HTML_TYPE_LINKTEXT:
-			ADD_SEP;
-			ADD_ITEM (_("Link..."), link_prop_dialog, LINK);
+			ADD_ITEM (_("Link Style..."), link_prop_dialog, LINK);
 			ADD_PROP (LINK);
-			ADD_ITEM (_("Paragraph..."), prop_dialog, PARAGRAPH);
+			ADD_ITEM (_("Paragraph Style..."), prop_dialog, PARAGRAPH);
 			ADD_PROP (PARAGRAPH);
 			break;
 		case HTML_TYPE_RULE:
-			ADD_SEP;
-			ADD_ITEM (_("Rule..."), prop_dialog, RULE);
+			ADD_ITEM (_("Rule Style..."), prop_dialog, RULE);
 			ADD_PROP (RULE);
 			break;
 		case HTML_TYPE_IMAGE:
-			ADD_SEP;
-			ADD_ITEM (_("Image..."), prop_dialog, IMAGE);
+			ADD_ITEM (_("Image Style..."), prop_dialog, IMAGE);
 			ADD_PROP (IMAGE);
-			ADD_ITEM (_("Paragraph..."), prop_dialog, PARAGRAPH);
+			ADD_ITEM (_("Paragraph Style..."), prop_dialog, PARAGRAPH);
 			ADD_PROP (PARAGRAPH);
 			break;
 		default:
+			;
 		}
 		if (obj->parent && obj->parent->parent && HTML_IS_TABLE_CELL (obj->parent->parent)) {
 			if (cd->format_html) {
-				ADD_SEP;
 				ADD_PROP (CELL);
-				ADD_ITEM (_("Cell..."), prop_dialog, CELL);
+				ADD_ITEM (_("Cell Style..."), prop_dialog, CELL);
 				if (obj->parent->parent->parent && HTML_IS_TABLE (obj->parent->parent->parent)) {
 					ADD_PROP (TABLE);
-					ADD_ITEM (_("Table..."), prop_dialog, TABLE);
+					ADD_ITEM (_("Table Style..."), prop_dialog, TABLE);
 				}
 			}
-			SUBMENU (N_("Table insert"));
+		}
+		if (cd->format_html) {
+			ADD_PROP (BODY);
+			ADD_ITEM (_("Page Style..."), prop_dialog, BODY);
+		}
+		END_SUBMENU;
+		ADD_SEP;
+		if (obj->parent && obj->parent->parent && HTML_IS_TABLE_CELL (obj->parent->parent)) {
+			SUBMENU (_("Table insert"));
 			ADD_ITEM (_("Table"), insert_table_cb, NONE);
 			ADD_SEP;
 			ADD_ITEM (_("Row above"), insert_row_above, NONE);
@@ -516,7 +523,7 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
 			ADD_ITEM (_("Column before"), insert_column_before, NONE);
 			ADD_ITEM (_("Column after"), insert_column_after, NONE);
 			END_SUBMENU;
-			SUBMENU (N_("Table delete"));
+			SUBMENU (_("Table delete"));
 			ADD_ITEM (_("Table"), delete_table, NONE);
 			ADD_ITEM (_("Row"), delete_row, NONE);
 			ADD_ITEM (_("Column"), delete_column, NONE);
@@ -525,13 +532,45 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
 		}
 	}
 
-	if (cd->format_html) {
+	if (!active && obj && html_object_is_text (obj)
+	    && !html_engine_spell_word_is_valid (e)) {
+		gchar *spell, *word, *ignore, *add;
+
 		ADD_SEP;
-		ADD_PROP (BODY);
-		ADD_ITEM (_("Page..."), prop_dialog, BODY);
+		ADD_ITEM (_("Check Word Spelling..."), spell_check_cb, NONE);
+		if (get_n_languages (cd) > 1) {
+			gchar *lang;
+			gint i;
+
+			SUBMENU (_("Add Word to"));
+
+			for (i = 0; i < cd->languages->_length; i ++) {
+				if (strstr (html_engine_get_language (cd->html->engine), cd->languages->_buffer [i].abbreviation)) {
+					lang = g_strdup_printf (_("%s Dictionary"), cd->languages->_buffer [i].name);
+					ADD_ITEM (lang, spell_add, NONE);
+					g_object_set_data (G_OBJECT (menuitem), "abbrev", cd->languages->_buffer [i].abbreviation);
+					g_free (lang);
+				}
+			}
+			
+			END_SUBMENU;
+		} else {
+			ADD_ITEM (_("Add Word to Dictionary"), spell_add, NONE);
+			g_object_set_data (G_OBJECT (menuitem), "abbrev", (gpointer) get_language (cd));
+		}
+		ADD_ITEM (_("Ignore Misspelled Word"), spell_ignore, NONE);
 	}
 
-	gtk_widget_show (menu);
+	SUBMENU (_("Input Methods"));
+	gtk_im_multicontext_append_menuitems (GTK_IM_MULTICONTEXT (cd->html->priv->im_context), 
+					      GTK_MENU_SHELL (menu));
+	END_SUBMENU;
+
+	if (*items == 0) {
+		gtk_object_sink (GTK_OBJECT (menu));
+		menu = NULL;
+	} else
+		gtk_widget_show (menu);
 
 	return menu;
 }
@@ -540,19 +579,19 @@ gint
 popup_show (GtkHTMLControlData *cd, GdkEventButton *event)
 {
 	GtkWidget *menu;
-	guint items = 0;
+	guint items, props;
+
+	menu = prepare_properties_and_menu (cd, &items, &props);
 
-	menu = prepare_properties_and_menu (cd, &items);
 	if (items)
-		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 
+		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
 				event ? event->button : 0, event ? event->time : 0);
-	gtk_widget_unref (menu);
 
 	return (items > 0);
 }
 
 static void
-set_position (GtkMenu *menu, gint *x, gint *y, gpointer data)
+set_position (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer data)
 {
 	GtkHTMLControlData *cd = (GtkHTMLControlData *) data;
 	HTMLEngine *e = cd->html->engine;
@@ -568,13 +607,12 @@ gint
 popup_show_at_cursor (GtkHTMLControlData *cd)
 {
 	GtkWidget *menu;
-	guint items = 0;
+	guint items, props;
 
-	menu = prepare_properties_and_menu (cd, &items);
+	menu = prepare_properties_and_menu (cd, &items, &props);
 	gtk_widget_show (menu);
 	if (items)
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, set_position, cd, 0, 0);
-	gtk_widget_unref (menu);
 
 	return (items > 0);
 }
@@ -582,9 +620,11 @@ popup_show_at_cursor (GtkHTMLControlData *cd)
 void
 property_dialog_show (GtkHTMLControlData *cd)
 {
-	guint items = 0;
+	GtkWidget *menu;
+	guint items, props;
 
-	gtk_widget_unref (prepare_properties_and_menu (cd, &items));
-	if (items)
+	menu = prepare_properties_and_menu (cd, &items, &props);
+	gtk_object_sink (GTK_OBJECT (menu));
+	if (props)
 		show_prop_dialog (cd, GTK_HTML_EDIT_PROPERTY_NONE);
 }
