@@ -21,6 +21,7 @@
 */
 
 #include <config.h>
+#include <libgnome/gnome-i18n.h>
 #include <gnome.h>
 #include <libgnomeui/gnome-window-icon.h>
 #include "replace.h"
@@ -72,10 +73,10 @@ ask_dialog_new (HTMLEngine *e)
 
 	d = g_new (GtkHTMLReplaceAskDialog, 1);
 	d->dialog = GTK_DIALOG (gtk_dialog_new_with_buttons (_("Replace confirmation"), NULL, 0,
-							     _("Replace all"), 1,
-							     _("Next"), 2,
+							     _("Replace _All"), 1,
+							     _("_Next"), 2,
 							     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-							     _("Replace"), 0,
+							     _("_Replace"), 0,
 							     NULL));
 	d->engine = e;
 
@@ -145,7 +146,7 @@ GtkHTMLReplaceDialog *
 gtk_html_replace_dialog_new (GtkHTML *html, GtkHTMLControlData *cd)
 {
 	GtkHTMLReplaceDialog *dialog = g_new (GtkHTMLReplaceDialog, 1);
-	GtkWidget *hbox;
+	GtkWidget *hbox, *vbox;
 	GtkWidget *table;
 	GtkWidget *label;
 
@@ -183,11 +184,15 @@ gtk_html_replace_dialog_new (GtkHTML *html, GtkHTMLControlData *cd)
 	gtk_box_pack_start (GTK_BOX (hbox), dialog->backward, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), dialog->case_sensitive, FALSE, FALSE, 0);
 
-	gtk_box_pack_start (GTK_BOX (dialog->dialog->vbox), table, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (dialog->dialog->vbox), hbox, FALSE, FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (dialog->dialog->vbox), 6);
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_widget_show (vbox);
+	gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog->dialog), 6);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog->dialog->vbox), 6);
+	gtk_box_set_spacing (GTK_BOX (dialog->dialog->vbox), 6);
+	gtk_box_pack_start (GTK_BOX (dialog->dialog->vbox), vbox, FALSE, FALSE, 0);
 	gtk_widget_show_all (table);
 	gtk_widget_show_all (hbox);
 
