@@ -20,6 +20,8 @@
     Boston, MA 02111-1307, USA.
 */
 
+#include <config.h>
+#include <libgnome/gnome-i18n.h>
 #include <string.h>
 #include <glade/glade.h>
 #include <gal/widgets/widget-color-combo.h>
@@ -37,7 +39,6 @@
 #include "htmltable.h"
 #include "htmlsettings.h"
 
-#include "config.h"
 #include "properties.h"
 #include "table.h"
 #include "utils.h"
@@ -524,7 +525,7 @@ table_widget (GtkHTMLEditTableProperties *d)
 	GtkWidget *table_page;
 	GladeXML *xml;
 
-	xml = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "table_page", NULL);
+	xml = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "table_page", GETTEXT_PACKAGE);
 	if (!xml)
 		g_error (_("Could not load glade file."));
 
@@ -578,7 +579,7 @@ table_widget (GtkHTMLEditTableProperties *d)
 	UPPER_FIX (cols);
 	UPPER_FIX (rows);
 
-	gtk_box_pack_start (GTK_BOX (table_page), sample_frame (&d->sample), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (table_page), sample_frame (&d->sample), TRUE, TRUE, 0);
 
 	gtk_widget_show_all (table_page);
         /* RM2 gdk_color_alloc (gdk_window_get_colormap (d->cd->html->engine->window), &d->bg_color); */
@@ -608,7 +609,7 @@ table_insert_widget (GtkHTMLEditTableProperties *d)
 	GladeXML *xml;
 
 	d->insert = TRUE;
-	xml       = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "table_insert_page", NULL);
+	xml       = glade_xml_new (GLADE_DATADIR "/gtkhtml-editor-properties.glade", "table_insert_page", GETTEXT_PACKAGE);
 	if (!xml)
 		g_error (_("Could not load glade file."));
 
@@ -644,7 +645,7 @@ table_insert_widget (GtkHTMLEditTableProperties *d)
 	g_signal_connect (gtk_option_menu_get_menu (GTK_OPTION_MENU (d->option_template)), "selection-done",
 			  G_CALLBACK (changed_template), d);
 	fill_templates (d);
-	gtk_box_pack_start (GTK_BOX (table_page), sample_frame (&d->sample), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (table_page), sample_frame (&d->sample), TRUE, TRUE, 0);
 
 	gtk_widget_show_all (table_page);
 
