@@ -21,8 +21,10 @@
 */
 
 #include <config.h>
-#include <gal/widgets/e-unicode.h>
+#include <libgnome/gnome-i18n.h>
+#include <string.h>
 #include <gal/widgets/widget-color-combo.h>
+
 #include "htmlcolor.h"
 #include "htmlcolorset.h"
 #include "htmlengine-edit.h"
@@ -31,6 +33,7 @@
 #include "htmlengine-save.h"
 #include "htmlselection.h"
 #include "htmlsettings.h"
+#include "htmltext.h"
 
 #include "text.h"
 #include "properties.h"
@@ -56,6 +59,8 @@ struct _GtkHTMLEditTextProperties {
 	gchar *url;
 
 	GtkHTML *sample;
+
+	HTMLText *text;
 };
 typedef struct _GtkHTMLEditTextProperties GtkHTMLEditTextProperties;
 
@@ -88,19 +93,21 @@ fill_sample (GtkHTMLEditTextProperties *d)
 	bg    = html_engine_save_get_sample_body (d->cd->html->engine, NULL);
 	sa    = d->url && *d->url ? "</a>" : "";
 	size  = g_strdup_printf ("<font size=%d>", get_size (d->style_or) + 1);
-	color = d->color_changed ? g_strdup_printf ("<font color=#%02x%02x%02x>",
-						    d->color->color.red   >> 8,
-						    d->color->color.green >> 8,
-						    d->color->color.blue  >> 8)
-		: g_strdup ("");
+	
+	color = g_strdup_printf ("<font color=#%02x%02x%02x>",
+				 d->color->color.red   >> 8,
+				 d->color->color.green >> 8,
+				 d->color->color.blue  >> 8);
+	
 	body  = g_strconcat (bg, a,
 			     CVAL (0) ? "<b>" : "",
 			     CVAL (1) ? "<i>" : "",
 			     CVAL (2) ? "<u>" : "",
 			     CVAL (3) ? "<s>" : "",
 			     size, color,
-			     "The quick brown fox jumps over the lazy dog.", sa, NULL);
-
+			     /* string marked for translations is sample text. you may want to translate it to common sample text in your language */
+			     _("The quick brown fox jumps over the lazy dog."), sa, NULL);
+	
 	gtk_html_load_from_string (d->sample, body, -1);
 	g_free (color);
 	g_free (size);
@@ -110,7 +117,8 @@ fill_sample (GtkHTMLEditTextProperties *d)
 }
 
 static void
-color_changed (GtkWidget *w, GdkColor *color, gboolean by_user, GtkHTMLEditTextProperties *data)
+color_changed (GtkWidget *w, GdkColor *color, gboolean custom, gboolean by_user, gboolean is_default,
+	       GtkHTMLEditTextProperties *data)
 {
 	html_color_unref (data->color);
 	data->color = color
@@ -126,7 +134,7 @@ color_changed (GtkWidget *w, GdkColor *color, gboolean by_user, GtkHTMLEditTextP
 static void
 set_size (GtkWidget *w, GtkHTMLEditTextProperties *data)
 {
-	gint size = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (w), "size"));
+	gint size = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (w), "size"));
 
 	data->style_and &= ~GTK_HTML_FONT_STYLE_SIZE_MASK;
 	data->style_or  &= ~GTK_HTML_FONT_STYLE_SIZE_MASK;
@@ -139,13 +147,15 @@ set_size (GtkWidget *w, GtkHTMLEditTextProperties *data)
 static void
 set_style (GtkWidget *w, GtkHTMLEditTextProperties *d)
 {
-	GtkHTMLFontStyle style = GPOINTER_TO_UINT (gtk_object_get_data (GTK_OBJECT (w), "style"));
+	GtkHTMLFontStyle style = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (w), "style"));
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w))) {
 		d->style_or  |= style;
 		d->style_and |= style;
-	} else
+	} else {
+		d->style_or  &= ~style;
 		d->style_and &= ~style;
+	}
 
 	d->style_changed = TRUE;
 	gtk_html_edit_properties_dialog_change (d->cd->properties_dialog);
@@ -164,7 +174,7 @@ static void
 set_url (GtkWidget *w, GtkHTMLEditTextProperties *data)
 {
 	g_free (data->url);
-	data->url = e_utf8_from_gtk_string (data->entry_url, gtk_entry_get_text (GTK_ENTRY (data->entry_url)));
+	data->url = g_strdup (gtk_entry_get_text (GTK_ENTRY (data->entry_url)));
 	data->url_changed = TRUE;
 
 	gtk_html_edit_properties_dialog_change (data->cd->properties_dialog);
@@ -178,6 +188,7 @@ text_properties (GtkHTMLControlData *cd, gpointer *set_data)
 	GtkWidget *vbox, *frame, *table, *menu, *menuitem, *hbox, *t1;
 	gboolean selection;
 	const gchar *target;
+	const gchar *url;
 	gint i;
 
 	selection = html_engine_is_selection_active (cd->html->engine);
@@ -191,27 +202,33 @@ text_properties (GtkHTMLControlData *cd, gpointer *set_data)
 	data->style_and       = GTK_HTML_FONT_STYLE_MAX;
 	data->style_or        = html_engine_get_font_style (cd->html->engine);
 	data->color           = html_engine_get_color (cd->html->engine);
+	data->text            = HTML_TEXT (cd->html->engine->cursor->object);
+
+	if (!data->color)
+		data->color = html_colorset_get_color (data->cd->html->engine->settings->color_set, 
+						       HTMLTextColor);
 
 	target = html_engine_get_target (cd->html->engine);
-	data->url             = selection ? g_strconcat (html_engine_get_url (cd->html->engine),
-							 target ? "#" : "", target, NULL) : NULL;
+	url  = html_engine_get_url (cd->html->engine);
+	data->url = selection ? g_strconcat (url ? url : "", target ? "#" : "", target, NULL) : NULL;
+
 	html_color_ref (data->color);
 
 	table = gtk_table_new (3, 2, FALSE);
-	gtk_container_border_width (GTK_CONTAINER (table), 3);
-	gtk_table_set_col_spacings (GTK_TABLE (table), 3);
-	gtk_table_set_row_spacings (GTK_TABLE (table), 2);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 12);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 4);
 
-	vbox = gtk_vbox_new (FALSE, 0);
+	vbox = gtk_vbox_new (FALSE, 6);
 	frame = gtk_frame_new (_("Style"));
 	t1 = gtk_table_new (2, 2, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (t1), 3);
+	gtk_container_set_border_width (GTK_CONTAINER (t1), 6);
 
 #define ADD_CHECK(x,c,r) \
 	data->check [i] = gtk_check_button_new_with_label (x); \
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->check [i]), data->style_or & styles [i]); \
-        gtk_object_set_data (GTK_OBJECT (data->check [i]), "style", GUINT_TO_POINTER (styles [i])); \
-        gtk_signal_connect (GTK_OBJECT (data->check [i]), "toggled", set_style, data); \
+        g_object_set_data (G_OBJECT (data->check [i]), "style", GUINT_TO_POINTER (styles [i])); \
+        g_signal_connect (data->check [i], "toggled", G_CALLBACK (set_style), data); \
 	gtk_table_attach (GTK_TABLE (t1), data->check [i], c, c + 1, r, r + 1, GTK_FILL | GTK_EXPAND, 0, 0, 0); \
         i++
 
@@ -230,19 +247,15 @@ text_properties (GtkHTMLControlData *cd, gpointer *set_data)
 		frame = gtk_frame_new (_("Click will follow this URL"));
 		data->entry_url = gtk_entry_new ();
 		if (data->url) {
-			gchar *url;
-
-			url = e_utf8_to_gtk_string (data->entry_url, data->url);
-			gtk_entry_set_text (GTK_ENTRY (data->entry_url), url);
-			g_free (url);
+			gtk_entry_set_text (GTK_ENTRY (data->entry_url), data->url);
 		}
 		f1 = gtk_frame_new (NULL);
-		gtk_container_set_border_width (GTK_CONTAINER (f1), 3);
+		gtk_container_set_border_width (GTK_CONTAINER (f1), 6);
 		gtk_frame_set_shadow_type (GTK_FRAME (f1), GTK_SHADOW_NONE);
 		gtk_container_add (GTK_CONTAINER (f1), data->entry_url);
 		gtk_container_add (GTK_CONTAINER (frame), f1);
 		gtk_box_pack_start_defaults (GTK_BOX (vbox), frame);
-		gtk_signal_connect (GTK_OBJECT (data->entry_url), "changed", GTK_SIGNAL_FUNC (set_url), data);
+		g_signal_connect (data->entry_url, "changed", G_CALLBACK (set_url), data);
 	}
 
 	gtk_table_attach_defaults (GTK_TABLE (table), vbox, 0, 1, 0, 2);
@@ -253,10 +266,10 @@ text_properties (GtkHTMLControlData *cd, gpointer *set_data)
 #undef ADD_ITEM
 #define ADD_ITEM(n) \
 	menuitem = gtk_menu_item_new_with_label (_(n)); \
-        gtk_menu_append (GTK_MENU (menu), menuitem); \
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem); \
         gtk_widget_show (menuitem); \
-        gtk_signal_connect (GTK_OBJECT (menuitem), "activate", GTK_SIGNAL_FUNC (set_size), data); \
-        gtk_object_set_data (GTK_OBJECT (menuitem), "size", GINT_TO_POINTER (i)); i++;
+        g_signal_connect (menuitem, "activate", G_CALLBACK (set_size), data); \
+        g_object_set_data (G_OBJECT (menuitem), "size", GINT_TO_POINTER (i)); i++;
 
 	i=GTK_HTML_FONT_STYLE_SIZE_1;
 	ADD_ITEM("-2");
@@ -271,20 +284,20 @@ text_properties (GtkHTMLControlData *cd, gpointer *set_data)
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (data->sel_size), menu);
 	gtk_option_menu_set_history (GTK_OPTION_MENU (data->sel_size), get_size (data->style_or));
 	vbox = gtk_vbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 3);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
 	gtk_box_pack_start (GTK_BOX (vbox), data->sel_size, FALSE, FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (frame), vbox);
 	gtk_table_attach (GTK_TABLE (table), frame, 1, 2, 0, 1, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
 
 	/* color selection */
 	frame = gtk_frame_new (_("Color"));
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 3);
+	hbox = gtk_hbox_new (FALSE, 12);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
+
 	data->color_combo = color_combo_new (NULL, _("Automatic"),
-					     &html_colorset_get_color (data->cd->html->engine->settings->color_set,
-								       HTMLTextColor)->color,
+					     &data->color->color,
 					     color_group_fetch ("text", data->cd));
-        gtk_signal_connect (GTK_OBJECT (data->color_combo), "changed", GTK_SIGNAL_FUNC (color_changed), data);
+        g_signal_connect (data->color_combo, "color_changed", G_CALLBACK (color_changed), data);
 
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), data->color_combo, FALSE, FALSE, 0);
@@ -302,36 +315,59 @@ text_properties (GtkHTMLControlData *cd, gpointer *set_data)
 	return table;
 }
 
-void
+gboolean
 text_apply_cb (GtkHTMLControlData *cd, gpointer get_data)
 {
 	GtkHTMLEditTextProperties *data = (GtkHTMLEditTextProperties *) get_data;
 
-	if (data->style_changed)
-		gtk_html_set_font_style (cd->html, data->style_and, data->style_or);
+	if (data->style_changed || data->url_changed || data->color_changed) {
+		HTMLEngine *e = cd->html->engine;
+		gint position;
 
-	if (data->url_changed) {
-		gchar *h;
+		position = e->cursor->position;
 
-		h = strchr (data->url, '#');
-		if (h) {
-			gchar *url;
+		if (!html_engine_is_selection_active (e) && e->cursor->object != HTML_OBJECT (data->text))
+			if (!html_cursor_jump_to (e->cursor, e, HTML_OBJECT (data->text), 1)) {
+				GtkWidget *dialog;
+				printf ("d: %p\n", data->cd->properties_dialog);
+				dialog = gtk_message_dialog_new (GTK_WINDOW (data->cd->properties_dialog->dialog),
+								 GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+								 _("The editted text was removed from the document.\nCannot apply your changes."));
+				gtk_dialog_run (GTK_DIALOG (dialog));
+				gtk_widget_destroy (dialog);
+				html_cursor_jump_to_position (e->cursor, e, position);
+				return FALSE;
+			}
 
-			url = alloca (h - data->url + 1);
-			url [h - data->url] = 0;
-			strncpy (url, data->url, h - data->url);
-			html_engine_edit_set_link (cd->html->engine, url, h);
-		} else {
-			html_engine_edit_set_link (cd->html->engine, data->url, NULL);
+		if (data->style_changed)
+			gtk_html_set_font_style (cd->html, data->style_and, data->style_or);
+
+		if (data->url_changed) {
+			gchar *h;
+
+			h = strchr (data->url, '#');
+			if (h) {
+				gchar *url;
+
+				url = alloca (h - data->url + 1);
+				url [h - data->url] = 0;
+				strncpy (url, data->url, h - data->url);
+				html_engine_edit_set_link (cd->html->engine, url, h);
+			} else {
+				html_engine_edit_set_link (cd->html->engine, data->url, NULL);
+			}
 		}
+
+		if (data->color_changed)
+			gtk_html_set_color (cd->html, data->color);
+
+		data->color_changed = FALSE;
+		data->style_changed = FALSE;
+		data->url_changed   = FALSE;
+		html_cursor_jump_to_position (e->cursor, e, position);
 	}
 
-	if (data->color_changed)
-		gtk_html_set_color (cd->html, data->color);
-
-	data->color_changed = FALSE;
-	data->style_changed = FALSE;
-	data->url_changed   = FALSE;
+	return TRUE;
 }
 
 void
