@@ -24,6 +24,7 @@
 /* FIXME: Should use BonoboUIHandler.  */
 
 #include <config.h>
+#include <libgnome/gnome-i18n.h>
 #include <gnome.h>
 #include <bonobo.h>
 #include <gal/widgets/widget-color-combo.h>
@@ -35,9 +36,6 @@
 #include "htmlengine-edit-fontstyle.h"
 #include "htmlsettings.h"
 
-#include "tt.xpm"
-
-
 #define EDITOR_TOOLBAR_PATH "/HTMLEditor"
 
 
@@ -91,7 +89,7 @@ paragraph_style_menu_item_activated_cb (GtkWidget *widget,
 	GtkHTML *html;
 
 	html = GTK_HTML (data);
-	style = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget), "paragraph_style_value"));
+	style = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "paragraph_style_value"));
 
 	/* g_warning ("Setting paragraph style to %d.", style); */
 
@@ -104,9 +102,9 @@ paragraph_style_menu_item_update (GtkWidget *widget, gpointer format_html)
 	GtkHTMLParagraphStyle style;
 	gint sensitive;
 
-	style = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (widget), "paragraph_style_value"));
+	style = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "paragraph_style_value"));
 	
-	sensitive = (format_html 
+	sensitive = (format_html
 		     || style == GTK_HTML_PARAGRAPH_STYLE_NORMAL
 		     || style == GTK_HTML_PARAGRAPH_STYLE_PRE
 		     || style == GTK_HTML_PARAGRAPH_STYLE_ITEMDOTTED
@@ -145,20 +143,15 @@ setup_paragraph_style_option_menu (GtkHTML *html)
 		menu_item = gtk_menu_item_new_with_label (_(paragraph_style_items[i].description));
 		gtk_widget_show (menu_item);
 
-		gtk_menu_append (GTK_MENU (menu), menu_item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 
-		gtk_object_set_data (GTK_OBJECT (menu_item), "paragraph_style_value",
+		g_object_set_data (G_OBJECT (menu_item), "paragraph_style_value",
 				     GINT_TO_POINTER (paragraph_style_items[i].style));
-		gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
-				    GTK_SIGNAL_FUNC (paragraph_style_menu_item_activated_cb),
-				    html);
+		g_signal_connect (menu_item, "activate", G_CALLBACK (paragraph_style_menu_item_activated_cb), html);
 	}
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
-
-	gtk_signal_connect (GTK_OBJECT (html), "current_paragraph_style_changed",
-			    GTK_SIGNAL_FUNC (paragraph_style_changed_cb), option_menu);
-
+	g_signal_connect (html, "current_paragraph_style_changed", G_CALLBACK (paragraph_style_changed_cb), option_menu);
 	gtk_widget_show (option_menu);
 
 	return option_menu;
@@ -167,7 +160,7 @@ setup_paragraph_style_option_menu (GtkHTML *html)
 static void
 set_font_size (GtkWidget *w, GtkHTMLControlData *cd)
 {
-	GtkHTMLFontStyle style = GTK_HTML_FONT_STYLE_SIZE_1 + GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (w),
+	GtkHTMLFontStyle style = GTK_HTML_FONT_STYLE_SIZE_1 + GPOINTER_TO_INT (g_object_get_data (G_OBJECT (w),
 												    "size"));
 
 	if (!cd->block_font_style_change)
@@ -207,19 +200,17 @@ setup_font_size_option_menu (GtkHTMLControlData *cd)
 		menu_item = gtk_menu_item_new_with_label (size);
 		gtk_widget_show (menu_item);
 
-		gtk_menu_append (GTK_MENU (menu), menu_item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 
-		gtk_object_set_data (GTK_OBJECT (menu_item), "size",
+		g_object_set_data (G_OBJECT (menu_item), "size",
 				     GINT_TO_POINTER (i));
-		gtk_signal_connect (GTK_OBJECT (menu_item), "activate",
-				    GTK_SIGNAL_FUNC (set_font_size), cd);
+		g_signal_connect (menu_item, "activate", G_CALLBACK (set_font_size), cd);
 	}
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
 	gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), 2);
 
-	gtk_signal_connect (GTK_OBJECT (cd->html), "insertion_font_style_changed",
-			    GTK_SIGNAL_FUNC (font_size_changed), cd);
+	g_signal_connect (cd->html, "insertion_font_style_changed", G_CALLBACK (font_size_changed), cd);
 
 	gtk_widget_show (option_menu);
 
@@ -244,15 +235,17 @@ void
 toolbar_apply_color (GtkHTMLControlData *cd)
 {
 	GdkColor *color;
+	gboolean default_color;
 
-	color = color_combo_get_color (COLOR_COMBO (cd->combo));
+	color = color_combo_get_color (COLOR_COMBO (cd->combo), &default_color);
 	apply_color (color, cd);
 	if (color)
 		gdk_color_free (color);
 }
 
 static void
-color_changed (GtkWidget *w, GdkColor *gdk_color, gboolean by_user, GtkHTMLControlData *cd)
+color_changed (GtkWidget *w, GdkColor *gdk_color, gboolean custom, gboolean by_user, gboolean is_default,
+	       GtkHTMLControlData *cd)
 {
 	/* If the color was changed programatically there's not need to set things */
 	if (!by_user)
@@ -279,7 +272,8 @@ static void
 realize_engine (GtkHTML *html, GtkHTMLControlData *cd)
 {
 	set_color_combo (html, cd);
-	gtk_signal_disconnect_by_func (GTK_OBJECT (html), realize_engine, cd);
+	g_signal_handlers_disconnect_matched (html, G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
+					      0, 0, NULL, G_CALLBACK (realize_engine), cd);
 }
 
 static void
@@ -288,7 +282,7 @@ load_done (GtkHTML *html, GtkHTMLControlData *cd)
 	if (GTK_WIDGET_REALIZED (cd->html))
 		set_color_combo (html, cd);
 	else
-		gtk_signal_connect (GTK_OBJECT (cd->html), "realize", realize_engine, cd);
+		g_signal_connect (cd->html, "realize", G_CALLBACK (realize_engine), cd);
 }
 
 static GtkWidget *
@@ -300,13 +294,11 @@ setup_color_combo (GtkHTMLControlData *cd)
 	if (GTK_WIDGET_REALIZED (cd->html))
 		html_color_alloc (color, cd->html->engine->painter);
 	else
-		gtk_signal_connect (GTK_OBJECT (cd->html), "realize", realize_engine, cd);
-        gtk_signal_connect (GTK_OBJECT (cd->html), "load_done", GTK_SIGNAL_FUNC (load_done), cd);
+		g_signal_connect (cd->html, "realize", G_CALLBACK (realize_engine), cd);
+        g_signal_connect (cd->html, "load_done", G_CALLBACK (load_done), cd);
 
 	cd->combo = color_combo_new (NULL, _("Automatic"), &color->color, color_group_fetch ("toolbar_text", cd));
-	GTK_WIDGET_UNSET_FLAGS (cd->combo, GTK_CAN_FOCUS);
-	gtk_container_forall (GTK_CONTAINER (cd->combo), unset_focus, NULL);
-        gtk_signal_connect (GTK_OBJECT (cd->combo), "changed", GTK_SIGNAL_FUNC (color_changed), cd);
+        g_signal_connect (cd->combo, "color_changed", G_CALLBACK (color_changed), cd);
 
 	gtk_widget_show_all (cd->combo);
 	return cd->combo;
@@ -465,9 +457,9 @@ safe_set_active (GtkWidget *widget,
 	object = GTK_OBJECT (widget);
 	toggle_button = GTK_TOGGLE_BUTTON (widget);
 
-	gtk_signal_handler_block_by_data (object, data);
+	g_signal_handlers_block_matched (object, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, data);
 	gtk_toggle_button_set_active (toggle_button, TRUE);
-	gtk_signal_handler_unblock_by_data (object, data);
+	g_signal_handlers_unblock_matched (object, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, data);
 }
 
 static void
@@ -513,26 +505,26 @@ editor_toolbar_unindent_cb (GtkWidget *widget,
 
 static GnomeUIInfo editor_toolbar_alignment_group[] = {
 	GNOMEUIINFO_ITEM_STOCK (N_("Left align"), N_("Left justifies the paragraphs"),
-				editor_toolbar_left_align_cb, GNOME_STOCK_PIXMAP_ALIGN_LEFT),
+				editor_toolbar_left_align_cb, GTK_STOCK_JUSTIFY_LEFT),
 	GNOMEUIINFO_ITEM_STOCK (N_("Center"), N_("Center justifies the paragraphs"),
-				editor_toolbar_center_cb, GNOME_STOCK_PIXMAP_ALIGN_CENTER),
+				editor_toolbar_center_cb, GTK_STOCK_JUSTIFY_CENTER),
 	GNOMEUIINFO_ITEM_STOCK (N_("Right align"), N_("Right justifies the paragraphs"),
-				editor_toolbar_right_align_cb, GNOME_STOCK_PIXMAP_ALIGN_RIGHT),
+				editor_toolbar_right_align_cb, GTK_STOCK_JUSTIFY_RIGHT),
 	GNOMEUIINFO_END
 };
 
 static GnomeUIInfo editor_toolbar_style_uiinfo[] = {
 
 	{ GNOME_APP_UI_TOGGLEITEM, N_("Typewriter"), N_("Toggle typewriter font style"),
-	  editor_toolbar_tt_cb, NULL, NULL, GNOME_APP_PIXMAP_DATA, tt_xpm },
+	  editor_toolbar_tt_cb, NULL, NULL, GNOME_APP_PIXMAP_FILENAME, GTKHTML_DATADIR "/icons/font-tt-24.png" },
 	{ GNOME_APP_UI_TOGGLEITEM, N_("Bold"), N_("Makes the text bold"),
-	  editor_toolbar_bold_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_TEXT_BOLD },
+	  editor_toolbar_bold_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_BOLD },
 	{ GNOME_APP_UI_TOGGLEITEM, N_("Italic"), N_("Makes the text italic"),
-	  editor_toolbar_italic_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_TEXT_ITALIC },
+	  editor_toolbar_italic_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_ITALIC },
 	{ GNOME_APP_UI_TOGGLEITEM, N_("Underline"), N_("Underlines the text"),
-	  editor_toolbar_underline_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_TEXT_UNDERLINE },
+	  editor_toolbar_underline_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_UNDERLINE },
 	{ GNOME_APP_UI_TOGGLEITEM, N_("Strikeout"), N_("Strikes out the text"),
-	  editor_toolbar_strikeout_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_TEXT_STRIKEOUT },
+	  editor_toolbar_strikeout_cb, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GTK_STOCK_STRIKETHROUGH },
 
 	GNOMEUIINFO_SEPARATOR,
 
@@ -541,9 +533,9 @@ static GnomeUIInfo editor_toolbar_style_uiinfo[] = {
 	GNOMEUIINFO_SEPARATOR,
 
 	GNOMEUIINFO_ITEM_STOCK (N_("Unindent"), N_("Indents the paragraphs less"),
-				editor_toolbar_unindent_cb, GNOME_STOCK_PIXMAP_TEXT_UNINDENT),
+				editor_toolbar_unindent_cb, GNOME_STOCK_TEXT_UNINDENT),
 	GNOMEUIINFO_ITEM_STOCK (N_("Indent"), N_("Indents the paragraphs more"),
-				editor_toolbar_indent_cb, GNOME_STOCK_PIXMAP_TEXT_INDENT),
+				editor_toolbar_indent_cb, GNOME_STOCK_TEXT_INDENT),
 
 	GNOMEUIINFO_END
 };
@@ -573,18 +565,13 @@ indentation_changed (GtkWidget *w, guint level, GtkHTMLControlData *cd)
 static GtkWidget *
 create_style_toolbar (GtkHTMLControlData *cd)
 {
-	GtkWidget *frame, *hbox;
-
+	GtkWidget *hbox;
+	gchar *domain;
+	
 	hbox = gtk_hbox_new (FALSE, 0);
-	frame = gtk_frame_new (NULL);
-	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
 
-	cd->toolbar_style = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
-
-	gtk_container_add (GTK_CONTAINER (frame), cd->toolbar_style);
-	gtk_box_pack_start (GTK_BOX (hbox), frame, TRUE, TRUE, 0);
-
-	gtk_widget_show_all (hbox);
+	cd->toolbar_style = gtk_toolbar_new ();
+	gtk_box_pack_start (GTK_BOX (hbox), cd->toolbar_style, TRUE, TRUE, 0);
 
 	cd->paragraph_option = setup_paragraph_style_option_menu (cd->html),
 	gtk_toolbar_prepend_widget (GTK_TOOLBAR (cd->toolbar_style),
@@ -595,14 +582,26 @@ create_style_toolbar (GtkHTMLControlData *cd)
 				    setup_font_size_option_menu (cd),
 				    NULL, NULL);
 
+	/* 
+	 * FIXME: steal textdomain temporarily from main-process,  and set it to 
+	 * GETTEXT_PACKAGE, after create the widgets, restore it.
+	 */
+	domain = g_strdup (textdomain (NULL));
+	textdomain (GETTEXT_PACKAGE);
+	
 	gnome_app_fill_toolbar_with_data (GTK_TOOLBAR (cd->toolbar_style), editor_toolbar_style_uiinfo, NULL, cd);
+
+	/* restore the stolen domain */
+	textdomain (domain);
+	g_free (domain);
+
 	gtk_toolbar_append_widget (GTK_TOOLBAR (cd->toolbar_style),
 				   setup_color_combo (cd),
 				   NULL, NULL);
 
 	cd->font_style_changed_connection_id
-		= gtk_signal_connect (GTK_OBJECT (cd->html), "insertion_font_style_changed",
-				      GTK_SIGNAL_FUNC (insertion_font_style_changed_cb), cd);
+		= g_signal_connect (GTK_OBJECT (cd->html), "insertion_font_style_changed",
+				      G_CALLBACK (insertion_font_style_changed_cb), cd);
 
 	/* The following SUCKS!  */
 	cd->tt_button        = editor_toolbar_style_uiinfo [0].widget;
@@ -616,36 +615,64 @@ create_style_toolbar (GtkHTMLControlData *cd)
 	cd->right_align_button = editor_toolbar_alignment_group[2].widget;
 
 	cd->unindent_button  = editor_toolbar_style_uiinfo [8].widget;
-	gtk_signal_connect (GTK_OBJECT (cd->html), "current_paragraph_indentation_changed",
-			    indentation_changed, cd);
+	gtk_widget_set_sensitive (cd->unindent_button, gtk_html_get_paragraph_indentation (cd->html) != 0);
+	g_signal_connect (cd->html, "current_paragraph_indentation_changed",
+			  G_CALLBACK (indentation_changed), cd);
 
 	cd->indent_button    = editor_toolbar_style_uiinfo [9].widget;
 
-	/* gtk_signal_connect (GTK_OBJECT (cd->html), "destroy",
-	   GTK_SIGNAL_FUNC (html_destroy_cb), cd);
+	/* g_signal_connect (GTK_OBJECT (cd->html), "destroy",
+	   G_CALLBACK (html_destroy_cb), cd);
 
-	   gtk_signal_connect (GTK_OBJECT (cd->toolbar_style), "destroy",
-	   GTK_SIGNAL_FUNC (toolbar_destroy_cb), cd); */
+	   g_signal_connect (GTK_OBJECT (cd->toolbar_style), "destroy",
+	   G_CALLBACK (toolbar_destroy_cb), cd); */
 
-	gtk_signal_connect (GTK_OBJECT (cd->html), "current_paragraph_alignment_changed",
-			    GTK_SIGNAL_FUNC (paragraph_alignment_changed_cb), cd);
+	g_signal_connect (cd->html, "current_paragraph_alignment_changed",
+			  G_CALLBACK (paragraph_alignment_changed_cb), cd);
+
+	gtk_toolbar_set_style (GTK_TOOLBAR (cd->toolbar_style), GTK_TOOLBAR_ICONS);
+	gtk_widget_show_all (hbox);
+
+	toolbar_update_format (cd);
+	GTK_WIDGET_UNSET_FLAGS (cd->toolbar_style, GTK_CAN_FOCUS);
+	gtk_container_forall (GTK_CONTAINER (cd->toolbar_style), unset_focus, NULL);
 
 	return hbox;
 }
 
+static gboolean
+toolbar_item_represents (GtkWidget *item, GtkWidget *widget)
+{
+	GtkWidget *parent;
+
+	if (item == widget)
+		return TRUE;
+
+	parent = gtk_widget_get_parent (widget);
+	while (parent) {
+		if (parent == item)
+			return TRUE;
+		parent = gtk_widget_get_parent (parent);
+	}
+
+	return FALSE;
+}
+
 static void
 toolbar_item_update_sensitivity (GtkWidget *widget, gpointer data)
 {
 	GtkHTMLControlData *cd = (GtkHTMLControlData *)data;
 	gboolean sensitive;
 
-	sensitive = (cd->format_html && widget != cd->unindent_button
-		     || widget == cd->paragraph_option
-		     || widget == cd->indent_button
-		     || widget == cd->unindent_button && gtk_html_get_paragraph_indentation (cd->html)
-		     || widget == cd->left_align_button
-		     || widget == cd->center_button
-		     || widget == cd->right_align_button);
+	if (toolbar_item_represents (widget, cd->unindent_button))
+		return;
+
+	sensitive = (cd->format_html
+		     || toolbar_item_represents (widget, cd->paragraph_option)
+		     || toolbar_item_represents (widget, cd->indent_button)
+		     || toolbar_item_represents (widget, cd->left_align_button)
+		     || toolbar_item_represents (widget, cd->center_button)
+		     || toolbar_item_represents (widget, cd->right_align_button));
 
 	gtk_widget_set_sensitive (widget, sensitive);
 }
@@ -653,11 +680,13 @@ toolbar_item_update_sensitivity (GtkWidget *widget, gpointer data)
 void
 toolbar_update_format (GtkHTMLControlData *cd)
 {
-	gtk_container_forall (GTK_CONTAINER (cd->toolbar_style), 
-			      toolbar_item_update_sensitivity, cd);
+	if (cd->toolbar_style)
+		gtk_container_foreach (GTK_CONTAINER (cd->toolbar_style), 
+		toolbar_item_update_sensitivity, cd);
 
-	paragraph_style_option_menu_set_mode (cd->paragraph_option, 
-					      cd->format_html);
+	if (cd->paragraph_option)
+		paragraph_style_option_menu_set_mode (cd->paragraph_option, 
+						      cd->format_html);
 }
 
 
