@@ -214,6 +214,11 @@ typedef enum {
 
 	GTK_HTML_COMMAND_SCROLL_BOD,
 	GTK_HTML_COMMAND_SCROLL_EOD,
+
+	GTK_HTML_COMMAND_CELL_CSPAN_INC,
+	GTK_HTML_COMMAND_CELL_RSPAN_INC,
+	GTK_HTML_COMMAND_CELL_CSPAN_DEC,
+	GTK_HTML_COMMAND_CELL_RSPAN_DEC,
 } GtkHTMLCommandType;
 
 typedef enum {
@@ -294,4 +299,8 @@ typedef enum {
 	GTK_HTML_STREAM_ERROR
 } GtkHTMLStreamStatus;
 
+typedef enum {
+	GTK_HTML_BEGIN_KEEP_SCROLL = 1 << 0,
+	GTK_HTML_BEGIN_KEEP_IMAGES = 1 << 1
+} GtkHTMLBeginFlags;
 #endif
