@@ -30,8 +30,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include <gal/widgets/e-unicode.h>
-
 #include "gtkhtml-properties.h"
 
 #include "htmlcolor.h"
@@ -51,14 +49,11 @@
 #include "htmltablecell.h"
 #include "htmltext.h"
 #include "htmltextslave.h"	/* FIXME */
-#include "htmlvspace.h"
 
-
 HTMLClueFlowClass html_clueflow_class;
 static HTMLClueClass *parent_class = NULL;
 
 #define HCF_CLASS(x) HTML_CLUEFLOW_CLASS (HTML_OBJECT (x)->klass)
-#define HTML_IS_PLAIN_PAINTER(obj)              (GTK_CHECK_TYPE ((obj), HTML_TYPE_PLAIN_PAINTER))
 
 inline HTMLHAlignType html_clueflow_get_halignment          (HTMLClueFlow *flow);
 static gchar *        get_item_number_str                   (HTMLClueFlow *flow);
@@ -66,11 +61,6 @@ static guint          get_post_padding                      (HTMLClueFlow *flow,
 							     guint pad);
 static int            get_similar_depth                     (HTMLClueFlow *self, 
 							     HTMLClueFlow *neighbor);
-
-
-#define CLUEFLOW_INDENT             "        "
-#define CLUEFLOW_BLOCKQUOTE_CITE    "> "
-          
 
 static void
 copy_levels (GByteArray *dst, GByteArray *src)
@@ -130,7 +120,8 @@ copy (HTMLObject *self,
 	HTML_CLUEFLOW (dest)->item_number = HTML_CLUEFLOW (self)->item_number;
 	HTML_CLUEFLOW (dest)->clear = HTML_CLUEFLOW (self)->clear;
 	HTML_CLUEFLOW (dest)->item_color = HTML_CLUEFLOW (self)->item_color;
-
+	HTML_CLUEFLOW (dest)->indent_width = HTML_CLUEFLOW (self)->indent_width;
+	
 	if (HTML_CLUEFLOW (dest)->item_color)
 		html_color_ref (HTML_CLUEFLOW (dest)->item_color);
 }
@@ -388,32 +379,18 @@ merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList **left, GList **
 static guint
 calc_padding (HTMLPainter *painter)
 {
-	guint ascent, descent;
-
-	/* FIXME maybe this should depend on the style.  */
-	/* FIXME move this into the painter.             */
-	if (HTML_IS_PLAIN_PAINTER (painter)) {
-		ascent = 0;
-		descent = 0;
-	} else {
-		ascent = html_painter_calc_ascent (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
-		descent = html_painter_calc_descent (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
+	if (!HTML_IS_PLAIN_PAINTER (painter)) {
+		return html_painter_get_space_width (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
 	}
-	return ascent + descent;
+	return 0;
 }
 
 static guint
 calc_bullet_size (HTMLPainter *painter)
 {
-	guint ascent, descent;
-
-	ascent = html_painter_calc_ascent (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
-	descent = html_painter_calc_descent (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
-
-	return (ascent + descent) / 3;
+	return html_painter_get_space_width (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL) / 2;
 }
 
-
 static gboolean
 is_cite (HTMLClueFlow *flow, gint level)
 {
@@ -422,7 +399,7 @@ is_cite (HTMLClueFlow *flow, gint level)
 
 	return FALSE;
 }
-
+  
 static gboolean
 is_header (HTMLClueFlow *flow)
 {
@@ -488,17 +465,19 @@ get_pre_padding (HTMLClueFlow *flow, guint pad)
 		return 0;
 
 	if (HTML_OBJECT_TYPE (prev_object) == HTML_TYPE_CLUEFLOW) {
-		HTMLClueFlow *prev = HTML_CLUEFLOW (prev_object);
+		HTMLClueFlow *prev;
 
-		if (get_post_padding (prev, 1))
+		if (get_post_padding (HTML_CLUEFLOW (prev_object), 1))
 			return 0;
-			
+
+		prev = HTML_CLUEFLOW (prev_object);
+
 		if (!is_levels_equal (flow, prev)) {
 			if (need_blockquote_padding (flow, prev))
 				return pad;
 			else 
 				return 0;
-		}
+  		}
 
 		if (flow->style == HTML_CLUEFLOW_STYLE_PRE
 		    && prev->style != HTML_CLUEFLOW_STYLE_PRE
@@ -585,50 +564,55 @@ get_level_indent (HTMLClueFlow *flow,
 		  gint level,
 		  HTMLPainter *painter)
 {
-	GtkHTMLFontStyle style;
-	int line_offset = 0;
 	guint indent = 0;
 	gint i = 0;
-	
-	style = html_clueflow_get_default_font_style (flow);
 
 	if (flow->levels->len > 0 || ! is_item (flow)) {
+		guint cite_width, indent_width;
+
+		cite_width   = html_painter_get_block_cite_width (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
+		indent_width = html_painter_get_block_indent_width (painter, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
+		
 		while (i <= level) {
 			switch (flow->levels->data[i]) {
 			case HTML_LIST_TYPE_BLOCKQUOTE_CITE:
-				indent += html_painter_calc_text_width (painter, 
-									CLUEFLOW_BLOCKQUOTE_CITE, 
-									strlen (CLUEFLOW_BLOCKQUOTE_CITE), &line_offset,
-									GTK_HTML_FONT_STYLE_SIZE_3, 
-									NULL);
+				indent += cite_width;
 				break;
 			case HTML_LIST_TYPE_GLOSSARY_DL:
 				indent += 0;
 				break;
 			default:
-				indent += html_painter_calc_text_width (painter, 
-									CLUEFLOW_INDENT, 
-									strlen (CLUEFLOW_INDENT), &line_offset,
-									GTK_HTML_FONT_STYLE_SIZE_3, 
-									NULL);
+				indent += indent_width;
 				break;
 			}
 			i++;
 		}
 	} else {
+		GtkHTMLFontStyle style;
+		style = html_clueflow_get_default_font_style (flow);
+
 		indent = 4 * html_painter_get_space_width (painter, style, NULL);
 	}
 
 	return indent;
 }
 
+static void
+set_painter (HTMLObject *o, HTMLPainter *painter)
+{
+	HTML_CLUEFLOW (o)->indent_width = -1;
+}
+
 static guint
 get_indent (HTMLClueFlow *flow,
 	    HTMLPainter *painter)
 {
-	return get_level_indent (flow, flow->levels->len - 1, painter);
-}
+	if (flow->indent_width < 0 )
+		flow->indent_width = get_level_indent (flow, flow->levels->len -1, painter);
 
+	return flow->indent_width;
+} 
+ 
 /* HTMLObject methods.  */
 
 static void
@@ -668,7 +652,7 @@ calc_min_width (HTMLObject *o,
 			w += add
 				? html_object_calc_preferred_width (cur, painter)
 				: html_object_calc_min_width (cur, painter);
-			if (!add || cur->flags & HTML_OBJECT_FLAG_NEWLINE || !cur->next) {
+			if (!add || !cur->next) {
 				if (min_width < w) min_width = w;
 				w = 0;
 			}
@@ -730,7 +714,7 @@ static gint
 object_nb_width (HTMLObject *o, HTMLPainter *painter, gboolean lineBegin)
 {
 	if (HTML_IS_TEXT_SLAVE (o))
-		return html_text_slave_nb_width (HTML_TEXT_SLAVE (o), painter, lineBegin);
+		return html_text_slave_get_nb_width (HTML_TEXT_SLAVE (o), painter, lineBegin);
 		
 	return html_object_calc_min_width (o, painter);
 }
@@ -869,8 +853,7 @@ layout_line (HTMLObject *o, HTMLPainter *painter, HTMLObject *begin,
 	x = start_lmargin = *lmargin;
 	o->ascent += o->y - old_y;
 
-	while (cur && !(cur->flags & HTML_OBJECT_FLAG_ALIGNED) && (HTML_CLUEFLOW (o)->style == HTML_CLUEFLOW_STYLE_PRE
-								   || (x < *rmargin || first))) {
+	while (cur && !(cur->flags & HTML_OBJECT_FLAG_ALIGNED)) {
 		HTMLFitType fit;
 		HTMLVAlignType valign;
 
@@ -894,6 +877,7 @@ layout_line (HTMLObject *o, HTMLPainter *painter, HTMLObject *begin,
 			need_update_height = TRUE;
 		}
 
+		cur->y = o->ascent + a;
 		fit = html_object_fit_line (cur, painter, first, first, FALSE, width_left (o, x, *rmargin));
 		first = FALSE;
 		if (fit == HTML_FIT_NONE)
@@ -978,6 +962,9 @@ calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 	oa = o->ascent;
 	od = o->descent;
 	ow = o->width;
+
+	cf->indent_width = -1;
+
 	o->ascent = 0;
 	o->descent = 0;
 	o->width = MAX (o->max_width, html_object_calc_min_width (o, painter));
@@ -1023,16 +1010,13 @@ calc_preferred_width (HTMLObject *o,
 	next = NULL;
 
 	for (obj = HTML_CLUE (o)->head; obj != 0; obj = obj->next) {
-		if (!(obj->flags & HTML_OBJECT_FLAG_NEWLINE)) {
-			w += html_object_calc_preferred_width (obj, painter);
-		}
+		w += html_object_calc_preferred_width (obj, painter);
 
-		if (obj->flags & HTML_OBJECT_FLAG_NEWLINE || !(next = html_object_next_not_slave (obj))) {
-			HTMLObject *eol = (obj->flags & HTML_OBJECT_FLAG_NEWLINE) ? html_object_prev_not_slave (obj) : obj;
+		if (!(next = html_object_next_not_slave (obj))) {
 
 			/* remove trailing space width on the end of line which is not on end of paragraph */
-			if (next && html_object_is_text (eol))
-				w -= html_text_trail_space_width (HTML_TEXT (eol), painter);
+			if (next && html_object_is_text (obj))
+				w -= html_text_trail_space_width (HTML_TEXT (obj), painter);
 
 			if (w > maxw)
 				maxw = w;
@@ -1125,22 +1109,27 @@ get_item_number_str (HTMLClueFlow *flow)
 static void
 draw_gt_line (HTMLObject *cur, HTMLPainter *p, gint offset, gint x, gint y)
 {
-	gint cy, a, d;
+	gint cy, w, a, d, line_offset = 0;
 
-	a = html_painter_calc_ascent (p, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
-	d = html_painter_calc_descent (p, GTK_HTML_FONT_STYLE_SIZE_3, NULL);
+	/* FIXME: cache items and glyphs? */
+	html_painter_calc_text_size (p, HTML_BLOCK_CITE, 
+				     strlen (HTML_BLOCK_CITE), NULL, NULL, 0, &line_offset,
+				     GTK_HTML_FONT_STYLE_SIZE_3, NULL,
+				     &w, &a, &d);
 
 	cy = offset;
 	while (cy + a <= cur->ascent) {
+		/* FIXME: cache items and glyphs? */
 		html_painter_draw_text (p, x, y + cur->y - cy,
-					CLUEFLOW_BLOCKQUOTE_CITE, 1, 0);
+					HTML_BLOCK_CITE, 1, NULL, NULL, 0, 0);
 		cy += a + d;
 	}
 
 	cy = - offset + a + d;
 	while (cy + d <= cur->descent) {
+		/* FIXME: cache items and glyphs? */
 		html_painter_draw_text (p, x, y + cur->y + cy,
-					CLUEFLOW_BLOCKQUOTE_CITE, 1, 0);
+					HTML_BLOCK_CITE, 1, NULL, NULL, 0, 0);
 		cy += a + d;
 	}
 }
@@ -1151,12 +1140,13 @@ draw_quotes (HTMLObject *self, HTMLPainter *painter,
 	     gint tx, gint ty)
 {
 	HTMLClueFlow *flow;
-	ArtIRect paint, area, clip;
+	GdkRectangle paint, area, clip;
 	int i;
 	int indent = 0;
 	int last_indent = 0;
 	gint pixel_size = html_painter_get_pixel_size (painter);
-
+	gboolean is_plain = HTML_IS_PLAIN_PAINTER (painter);
+	
 	flow = HTML_CLUEFLOW (self);
 
 	for (i = 0; i < flow->levels->len; i++, last_indent = indent) {
@@ -1164,24 +1154,23 @@ draw_quotes (HTMLObject *self, HTMLPainter *painter,
 
 		html_painter_set_pen (painter, &html_colorset_get_color_allocated (painter, HTMLLinkColor)->color);
 		if (is_cite (flow, i)) {
-			if (!HTML_IS_PLAIN_PAINTER (painter)) {
-				area.x0 = self->x + indent - 5 * pixel_size;
-				area.x1 = area.x0 + 2 * pixel_size;
-				area.y0 = self->y - self->ascent;
-				area.y1 = self->y + self->descent;
+			if (!is_plain) {
+				area.x = self->x + indent - 5 * pixel_size;
+				area.width = 2 * pixel_size;
+				area.y = self->y - self->ascent;
+				area.height = self->ascent + self->descent;
 				
-				clip.x0 = x;
-				clip.x1 = x + width;
-				clip.y0 = y;
-				clip.y1 = y + height;
+				clip.x = x;
+				clip.width = width;
+				clip.y = y;
+				clip.height = height;
 				
-				art_irect_intersect (&paint, &clip, &area);
-				if (art_irect_empty (&paint))
+				if (!gdk_rectangle_intersect (&clip, &area, &paint))
 					return;
-				
+
 				html_painter_fill_rect (painter, 
-							paint.x0 + tx, paint.y0 + ty,
-							paint.x1 - paint.x0, paint.y1 - paint.y0);
+							paint.x + tx, paint.y + ty,
+							paint.width, paint.height);
 			} else {
 				HTMLObject *cur = HTML_CLUE (self)->head;
 				gint baseline = 0;
@@ -1189,13 +1178,15 @@ draw_quotes (HTMLObject *self, HTMLPainter *painter,
 					if (cur->y != 0) {
 						baseline = cur->y;
 						break;
-					}
-					cur = cur->next;
-				}
+  					}
+  					cur = cur->next;
+  				}
+
+
 				/* draw "> " quote characters in the plain case */ 
 				html_painter_set_font_style (painter, 
 							     html_clueflow_get_default_font_style (flow));
-				
+						
 				html_painter_set_font_face  (painter, NULL);
 				draw_gt_line (self, painter, self->ascent - baseline,
 					      self->x + tx + last_indent, ty);
@@ -1223,13 +1214,13 @@ draw_item (HTMLObject *self, HTMLPainter *painter, gint x, gint y, gint width, g
 	} else
 		html_painter_set_pen (painter, &html_colorset_get_color_allocated (painter, HTMLTextColor)->color);
 
-	indent = get_level_indent (flow, flow->levels->len - 1, painter);
+	indent = get_indent (flow, painter);
 	if (flow->item_type == HTML_LIST_TYPE_UNORDERED) {
 		guint bullet_size;
 		gint xp, yp;
 		bullet_size = MAX (3, calc_bullet_size (painter));
 
-		xp = self->x + indent - 2 * bullet_size;
+		xp = self->x + indent - 2 * bullet_size;	
 		yp = self->y - self->ascent 
 			+ (first->y - first->ascent) 
 			+ (first->ascent + first->descent)/2 
@@ -1251,17 +1242,19 @@ draw_item (HTMLObject *self, HTMLPainter *painter, gint x, gint y, gint width, g
 
 		number = get_item_number_str (flow);
 		if (number) {
-			gint width, len, line_offset = 0;
+			gint width, len, line_offset = 0, asc, dsc;
 
 			len   = strlen (number);
-			width = html_painter_calc_text_width (painter, number, len, &line_offset,
-							      html_clueflow_get_default_font_style (flow), NULL);
-
+			/* FIXME: cache items and glyphs? */
+			html_painter_calc_text_size (painter, number, len, NULL, NULL, 0, &line_offset,
+						     html_clueflow_get_default_font_style (flow), NULL, &width, &asc, &dsc);
+			width += html_painter_get_space_width (painter, html_clueflow_get_default_font_style (flow), NULL);
 			html_painter_set_font_style (painter, html_clueflow_get_default_font_style (flow));
 			html_painter_set_font_face  (painter, NULL);
-			html_painter_draw_text (painter, self->x + indent - width + tx,
+			/* FIXME: cache items and glyphs? */
+			html_painter_draw_text (painter, self->x + first->x - width + tx,
 						self->y - self->ascent + first->y + ty,
-						number, strlen (number), 0);
+						number, strlen (number), NULL, NULL, 0, 0);
 		}
 		g_free (number);
 	}
@@ -1327,7 +1320,8 @@ check_point (HTMLObject *self,
 	 * shift any selection inside the indent block to the 
 	 * left edge of the flow.
 	 */
-	x = MAX (x, get_indent (HTML_CLUEFLOW (self), painter));
+	if (for_cursor)
+		x = MAX (x, get_indent (HTML_CLUEFLOW (self), painter));
 
 	for (p = clue->head; p; ) {
 
@@ -1705,24 +1699,6 @@ save (HTMLObject *s,
 	return TRUE;
 }
 
-static gint
-string_append_nonbsp (GString *out, guchar *s, gint length)
-{
-	gint len = length;
-
-	while (len--) {
-		if (IS_UTF8_NBSP (s)) {
-			g_string_append_c (out, ' ');
-			s += 2;
-			len--;
-		} else {
-			g_string_append_c (out, *s);
-			s++;
-		}
-	}
-	return length;
-}
-
 static void
 write_item_marker (GString *pad_string, HTMLClueFlow *flow)
 {
@@ -1779,12 +1755,12 @@ plain_padding (HTMLClueFlow *flow, GString *out, gboolean firstline)
 		for (i = 0; i < flow->levels->len; i++) {
 			switch (flow->levels->data[i]) {
 			case HTML_LIST_TYPE_BLOCKQUOTE_CITE:
-				APPEND_PLAIN (CLUEFLOW_BLOCKQUOTE_CITE);
+				APPEND_PLAIN (HTML_BLOCK_CITE);
 				break;
 			case HTML_LIST_TYPE_GLOSSARY_DL:
 				break;
 			default:
-				APPEND_PLAIN (CLUEFLOW_INDENT);
+				APPEND_PLAIN (HTML_BLOCK_INDENT);
 				break;
 			}
 		}
@@ -1894,7 +1870,7 @@ save_plain (HTMLObject *self,
 				align_pad--;
 			}
 
-			s += string_append_nonbsp (out, s, len);
+			s += html_engine_save_string_append_nonbsp (out, s, len);
 			
 			/* Trim the space at the end */
 			while (*s == ' ' || IS_UTF8_NBSP (s)) 
@@ -1962,36 +1938,34 @@ get_default_font_style (const HTMLClueFlow *self)
 static void
 search_set_info (HTMLObject *cur, HTMLSearch *info, guchar *text, guint pos, guint len)
 {
-	guint text_len = 0;
+	guint text_len = (info->forward) ? 0 : strlen(text);
 	guint cur_len;
-
 	info->found_len = len;
 
 	if (info->found) {
 		g_list_free (info->found);
 		info->found = NULL;
 	}
-
 	while (cur) {
 		if (html_object_is_text (cur)) {
 			cur_len = strlen (HTML_TEXT (cur)->text);
-			if (text_len + cur_len > pos) {
+			if (info->forward ? text_len + cur_len > pos : text_len - cur_len <= pos ) {
 				if (!info->found) {
-					info->start_pos = g_utf8_pointer_to_offset (text + text_len,
+					info->start_pos = g_utf8_pointer_to_offset (text + text_len - (info->forward ? 0 : cur_len),
 										    text + pos);
 				}
 				info->found = g_list_append (info->found, cur);
-			}
-			text_len += cur_len;
-			if (text_len >= pos+info->found_len) {
-				info->stop_pos = info->start_pos + info->found_len;
+			} 
+			text_len += (info->forward ? cur_len : -cur_len);
+			if (info->forward ? (text_len >= pos + info->found_len ): text_len < pos + info->found_len) {
+				info->stop_pos = info->start_pos + info->text_len;
 				info->last     = HTML_OBJECT (cur);
 				return;
 			}
 		} else if (HTML_OBJECT_TYPE (cur) != HTML_TYPE_TEXTSLAVE) {
 			break;
 		}		
-		cur = cur->next;
+		cur = info->forward ? cur->next : cur->prev;
 	}
 
 	g_assert_not_reached ();
@@ -2008,10 +1982,10 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 	guint text_len;
 	guint eq_len;
 	gint  pos;
+	guint utf8_len; 
 	gboolean retval = FALSE;
 
 	/* printf ("search flow look for \"text\" %s\n", info->text); */
-
 	/* first get flow text_len */
 	text_len = 0;
 	while (cur) {
@@ -2023,16 +1997,14 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 		}
 		cur = (info->forward) ? cur->next : cur->prev;
 	}
-
+	
 	if (text_len > 0) {
-		par = g_new (gchar, text_len+1);
-		par [text_len] = 0;
+		par = g_new0 (gchar, text_len+1);
 
 		pp = (info->forward) ? par : par+text_len;
 
 		/* now fill par with text */
-		head = cur = (info->forward) ? *beg : end;
-		cur = *beg;
+		head = cur = *beg;
 		while (cur) {
 			if (html_object_is_text (cur)) {
 				if (!info->forward) {
@@ -2050,24 +2022,49 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 
 		/* set eq_len and pos counters */
 		eq_len = 0;
+		utf8_len = g_utf8_strlen (par, -1);
 		if (info->found) {
-			pos = info->start_pos + ((info->forward) ? 1 : -1);
+			if (info->forward) {
+				pos = info->start_pos + 1 ;
+				/* convert pos into offset within par */
+				pos = (gint)g_utf8_offset_to_pointer (par, pos) - (gint)par;
+			} else {
+				gint head_len = g_utf8_strlen (((HTMLText*)head)->text, -1);
+				/* New starting pos for backward search */
+				/* within the (head)->text		*/
+
+				pos = info->stop_pos - 2;
+
+				/* Absolute position within par */
+				if (pos >= 0)
+					pos = utf8_len - (head_len - pos);
+				
+				/* Get the byte offset with par, skip if */
+				/* par and (head)->text are the same     */
+
+				if (utf8_len >= head_len + 1)
+					pos = (gint) g_utf8_offset_to_pointer (par, pos < 0 ? utf8_len - head_len - 1 : pos) - (gint) par;
+
+				/* We have now the starting byte of the char */
+				/* from which the backward search starts, get*/
+				/* the end byte like follows		     */		
+				pos = (gint) g_utf8_next_char (par + pos) - (gint) par - 1;
+			}
 		} else {
-			pos = (info->forward) ? 0 : text_len - 1;
+			/* Last char in par, from which a new backward */
+			/* search  starts 			       */
+
+			gint tot_len = strlen (par);
+			pos = (info->forward) ? 0 : tot_len - 1;
 		}
-
-		/* FIXME make shorter text instead */
-		if (!info->forward)
-			par [pos+1] = 0;
-
 		if ((info->forward && pos < text_len)
-		    || (!info->forward && pos>0)) {
+		    || (!info->forward && pos>=0)) {
 			if (info->reb) {
 				/* regex search */
 				gint rv;
 #ifndef HAVE_GNU_REGEX
 				regmatch_t match;
-				guchar *p=par+pos;
+				/* guchar *p=par+pos; */
 
 				/* FIXME UTF8
 				   replace &nbsp;'s with spaces
@@ -2109,17 +2106,18 @@ search_text (HTMLObject **beg, HTMLSearch *info)
 				}
 #endif
 			} else {
+				gint info_len = strlen (info->text);
 				/* substring search - simple one - could be improved
 				   go thru par and look for info->text */
 				while (par [pos]) {
 					if (info->trans [(guchar) info->text
-							[(info->forward) ? eq_len : info->text_len - eq_len - 1]]
+							[(info->forward) ? eq_len : info_len - eq_len - 1]]
 					    == info->trans [par [pos]]) {
 						eq_len++;
-						if (eq_len == info->text_len) {
+						if (eq_len == info_len) {
 							search_set_info (head, info, par,
 									 pos - (info->forward ? eq_len-1 : 0),
-									 info->text_len);
+									 info_len);
 							retval=TRUE;
 							break;
 						}
@@ -2162,20 +2160,31 @@ search (HTMLObject *obj, HTMLSearch *info)
 		}
 	}
 	while (cur) {
+		gboolean found = FALSE;
+		gboolean is_text;
+
+		is_text = html_object_is_text (cur);
+
 		if (html_object_is_text (cur)) {
 			if (search_text (&cur, info))
 				return TRUE;
-		} else {
-			html_search_push (info, cur);
-			if (html_object_search (cur, info))
-				return TRUE;
-			html_search_pop (info);
-			cur = (info->forward) ? cur->next : cur->prev;
 		}
+
 		if (info->found) {
 			g_list_free (info->found);
 			info->found = NULL;
 			info->start_pos = 0;
+			found = TRUE;
+		}
+
+		if (!is_text) {
+			if (!found || (info->start_pos < 0 && info->forward) || (info->start_pos >= 0 && !info->forward)) {
+				html_search_push (info, cur);
+				if (html_object_search (cur, info))
+					return TRUE;
+				html_search_pop (info);
+			}
+			cur = (info->forward) ? cur->next : cur->prev;
 		}
 	}
 
@@ -2203,7 +2212,7 @@ relayout (HTMLObject *self,
 	if (mw <= self->max_width)
 		return (*HTML_OBJECT_CLASS (parent_class)->relayout) (self, engine, child);
 	html_engine_calc_size (engine, FALSE);
-	html_engine_draw (engine, 0, 0, engine->width, engine->height);
+	html_engine_draw (engine, engine->x_offset, engine->y_offset, engine->width, engine->height);
 
 	return TRUE;
 }
@@ -2250,6 +2259,7 @@ html_clueflow_class_init (HTMLClueFlowClass *klass,
 	object_class->relayout = relayout;
 	object_class->get_recursive_length = get_recursive_length;
 	object_class->get_clear = get_clear;
+	object_class->set_painter = set_painter;
 
 	klass->get_default_font_style = get_default_font_style;
 
@@ -2259,7 +2269,7 @@ html_clueflow_class_init (HTMLClueFlowClass *klass,
 void
 html_clueflow_init (HTMLClueFlow *clueflow, HTMLClueFlowClass *klass,
 		    HTMLClueFlowStyle style, GByteArray *levels, HTMLListType item_type, gint item_number,
-		    HTMLClearType clear)
+ 		    HTMLClearType clear)
 {
 	HTMLObject *object;
 	HTMLClue *clue;
@@ -2276,6 +2286,7 @@ html_clueflow_init (HTMLClueFlow *clueflow, HTMLClueFlowClass *klass,
 
 	clueflow->style = style;
 	clueflow->levels = levels; 
+	clueflow->indent_width = -1;
 
 	clueflow->item_type   = item_type;
 	clueflow->item_number = item_number;
@@ -2285,8 +2296,7 @@ html_clueflow_init (HTMLClueFlow *clueflow, HTMLClueFlowClass *klass,
 }
 
 HTMLObject *
-html_clueflow_new (HTMLClueFlowStyle style, GByteArray *levels,
-		   HTMLListType item_type, gint item_number, HTMLClearType clear)
+html_clueflow_new (HTMLClueFlowStyle style, GByteArray *levels, HTMLListType item_type, gint item_number, HTMLClearType clear)
 {
 	HTMLClueFlow *clueflow;
 
@@ -2301,7 +2311,7 @@ html_clueflow_new_from_flow (HTMLClueFlow *flow)
 {
 	HTMLObject *o;
 
-	o = html_clueflow_new (flow->style, html_clueflow_dup_levels (flow),
+	o = html_clueflow_new (flow->style, html_clueflow_dup_levels (flow), 
 			       flow->item_type, flow->item_number, flow->clear);
 	html_object_copy_data_from_object (o, HTML_OBJECT (flow));
 
@@ -2632,7 +2642,6 @@ html_clueflow_get_properties (HTMLClueFlow *flow,
 		*alignment_return = HTML_CLUE (flow)->halign;
 }
 #endif
-
 /* spell checking */
 
 #include "htmlinterval.h"
@@ -2761,13 +2770,16 @@ static gchar *
 end_of_word (gchar *ct, gboolean cited)
 {
 	gunichar uc, ucn;
-	gchar *cn, *cnn;
+	gchar *cn;
 	gboolean cited2;
 
 	cited2 = FALSE;
-	while (*ct && (cn = e_unicode_get_utf8 (ct, &uc))
+	while (*ct
+	       && (uc = g_utf8_get_char (ct))
+	       && (cn = g_utf8_next_char (ct))
 	       && (html_selection_spell_word (uc, &cited2)
-		   || (!cited && cited2 && (cnn = e_unicode_get_utf8 (cn, &ucn)) && g_unichar_isalpha (ucn)))) {
+		   || (!cited && cited2)
+		   || (cited && cited2 && (ucn = g_utf8_get_char (cn)) && g_unichar_isalpha (ucn)))) {
 		ct = cn;
 		cited2 = FALSE;
 	}
@@ -2798,9 +2810,7 @@ html_clueflow_spell_check (HTMLClueFlow *flow, HTMLEngine *e, HTMLInterval *inte
 	   printf ("html_clueflow_spell_check %p %p %d %d\n", i->from, i->to, i->from_offset, i->to_offset); */
 
 	clue = HTML_CLUE (flow);
-	if (!e->widget->editor_api
-	    || !GTK_HTML_CLASS (GTK_OBJECT (e->widget)->klass)->properties->live_spell_check
-	    || !clue || !clue->tail)
+	if (!e->widget->editor_api || !gtk_html_get_inline_spelling (e->widget) || !clue || !clue->tail)
 		return;
 
 	off  = 0;
@@ -2890,16 +2900,19 @@ html_clueflow_get_line_offset (HTMLClueFlow *flow, HTMLPainter *painter, HTMLObj
 	o = head = child;
 	while (o) {
 		o = head->prev;
-		if (o && o->y + o->descent - 1 < child->y - child->ascent)
-			break;
-		else
-			head = o;
+		if (o) {
+			if (o->y + o->descent - 1 < child->y - child->ascent)
+				break;
+			else
+				head = o;
+		}
 	}
 
 	if (HTML_IS_TEXT_SLAVE (head)) {
 		HTMLTextSlave *bol = HTML_TEXT_SLAVE (head);
-		line_offset = html_text_text_line_length (html_text_get_text (bol->owner, bol->posStart),
-							  0, bol->owner->text_len - bol->posStart);
+
+		html_text_text_line_length (html_text_get_text (bol->owner, bol->posStart),
+					    &line_offset, bol->owner->text_len - bol->posStart, NULL);
 		head = html_object_next_not_slave (head);
 	}
 
@@ -2907,7 +2920,7 @@ html_clueflow_get_line_offset (HTMLClueFlow *flow, HTMLPainter *painter, HTMLObj
 		if (head == child)
 			break;
 		line_offset += html_object_get_line_length (head, painter, line_offset);
-		head = html_object_prev_not_slave (head);
+		head = html_object_next_not_slave (head);
 	}
 	/* printf ("lo: %d\n", line_offset); */
 	return line_offset;
