@@ -44,7 +44,7 @@ copy (HTMLObject *self,
 {
 	(* HTML_OBJECT_CLASS (parent_class)->copy) (self, dest);
 
-	g_warning ("HTMLEmbedded::copy is broken.");
+	/* FIXME g_warning ("HTMLEmbedded::copy is not complete."); */
 
 	HTML_EMBEDDED (dest)->name = g_strdup (HTML_EMBEDDED (self)->name);
 	HTML_EMBEDDED (dest)->value = g_strdup (HTML_EMBEDDED (self)->value);
@@ -71,13 +71,13 @@ draw (HTMLObject *o,
 		return;
 
 	if (element->parent) {
-		new_x = GTK_LAYOUT (element->parent)->hadjustment->value + o->x + tx;
-		new_y = GTK_LAYOUT (element->parent)->vadjustment->value + o->y + ty - o->ascent;
+		new_x = o->x + tx;
+		new_y = o->y + ty - o->ascent;
 		
 		if (element->widget->parent) {
 			if (new_x != element->abs_x || new_y != element->abs_y)
 				gtk_layout_move (GTK_LAYOUT(element->parent), element->widget, new_x, new_y);
-			else
+			else if (!GTK_HTML (element->parent)->engine->expose)
 				gtk_widget_queue_draw (element->widget);
 		}
 	
@@ -104,10 +104,10 @@ destroy (HTMLObject *o)
 		g_free(element->value);
 	if(element->widget) {
 		gtk_widget_hide (element->widget);
-		gtk_signal_disconnect_by_data (GTK_OBJECT (element->widget), element);
+		g_signal_handlers_disconnect_matched (element->widget, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, element);
 		if (element->changed_id > 0)
-			gtk_signal_disconnect (GTK_OBJECT (element->widget), element->changed_id);
-		gtk_object_set_data (GTK_OBJECT (element->widget), "embeddedelement", NULL);
+			g_signal_handler_disconnect (element->widget, element->changed_id);
+		g_object_set_data (G_OBJECT (element->widget), "embeddedelement", NULL);
 		if (element->widget->parent && element->parent) {
 			g_assert (element->widget->parent == element->parent);
 			gtk_container_remove (GTK_CONTAINER (element->parent), element->widget);
@@ -188,16 +188,6 @@ accepts_cursor (HTMLObject *o)
 	return TRUE;
 }
 
-static void
-reparent (HTMLEmbedded *e, GtkWidget *new_parent)
-{
-	e->parent = new_parent;
-	gtk_widget_ref (e->widget);
-	gtk_widget_unparent (e->widget);
-	gtk_layout_put (GTK_LAYOUT(e->parent), e->widget, 0, 0);
-	gtk_widget_unref (e->widget);
-}
-
 static gchar *
 encode (HTMLEmbedded *e)
 {
@@ -229,7 +219,7 @@ html_embedded_set_form (HTMLEmbedded *e, HTMLForm *form)
 }
 
 gchar *
-html_embedded_encode_string (gchar *str)
+html_embedded_encode_string (const gchar *str)
 {
         static gchar *safe = "$-._!*(),"; /* RFC 1738 */
         unsigned pos = 0;
@@ -293,7 +283,6 @@ html_embedded_class_init (HTMLEmbeddedClass *klass,
 	/* HTMLEmbedded methods.   */
 	klass->reset = reset;
 	klass->encode = encode;
-	klass->reparent = reparent;
 
 	/* HTMLObject methods.   */
 	object_class->destroy = destroy;
@@ -349,7 +338,7 @@ html_embedded_object_changed (GtkHTMLEmbedded *eb, HTMLEngine *e)
 {
 	HTMLObject *object;
 
-	object = gtk_object_get_data(GTK_OBJECT(eb), "embeddedelement");
+	object = HTML_OBJECT (g_object_get_data (G_OBJECT (eb), "embeddedelement"));
 	if (object)
 		html_object_calc_size (object, e->painter, FALSE);
 	
@@ -368,9 +357,8 @@ html_embedded_new_widget (GtkWidget *parent, GtkHTMLEmbedded *eb, HTMLEngine *en
 	/* pass em as the user_data so that the handler will disconnect 
 	 * when the object is destoyed
 	 */
-	gtk_signal_connect(GTK_OBJECT(eb), "button_press_event",
-			   GTK_SIGNAL_FUNC(html_embedded_grab_cursor), em);
-	em->changed_id = gtk_signal_connect (GTK_OBJECT (eb), "changed", html_embedded_object_changed, engine);
+	g_signal_connect(eb, "button_press_event", G_CALLBACK (html_embedded_grab_cursor), em);
+	em->changed_id = g_signal_connect (eb, "changed", G_CALLBACK (html_embedded_object_changed), engine);
 	/* printf ("id %u\n", em->changed_id); */
 
 	return em;
@@ -397,10 +385,9 @@ html_embedded_set_widget (HTMLEmbedded *emb, GtkWidget *w)
 	emb->widget = w;
 	
 	gtk_widget_show (w);
-	
-	gtk_object_set_data (GTK_OBJECT (w), "embeddedelement", emb);
-	gtk_signal_connect (GTK_OBJECT (w), "size_allocate",
-			    GTK_SIGNAL_FUNC (html_embedded_allocate), emb);
+
+	g_object_set_data (G_OBJECT (w), "embeddedelement", emb);
+	g_signal_connect (w, "size_allocate", G_CALLBACK (html_embedded_allocate), emb);
 }
 
 GtkWidget *
