@@ -876,8 +876,7 @@ insert_object_do (HTMLEngine *e, HTMLObject *obj, guint *len, gint level, guint
 	printf ("position after merge %d\n", e->cursor->position);
 #endif
 
-	html_cursor_destroy (e->cursor);
-	e->cursor = html_cursor_dup (orig);
+	html_cursor_copy (e->cursor, orig);
 	html_cursor_jump_to_position_no_spell (e->cursor, e, position_after);
 
 	if (check)
@@ -1119,7 +1118,8 @@ static void
 check_magic_link (HTMLEngine *e, const gchar *text, guint len)
 {
 	if (HTML_IS_TEXT (e->cursor->object)
-	    && GTK_HTML_PROPERTY (e->widget, magic_links) && len == 1
+	    && gtk_html_get_magic_links (e->widget) 
+	    && len == 1
 	    && (*text == ' ' || text [0] == '\n' || text [0] == '>' || text [0] == ')'))
 		html_text_magic_link (HTML_TEXT (e->cursor->object), e, html_object_get_length (e->cursor->object));
 }
@@ -1223,7 +1223,7 @@ use_pictograms (HTMLEngine *e)
 	gint relative;
 	gunichar uc;
 
-	if (!html_object_is_text (e->cursor->object))
+	if (!html_object_is_text (e->cursor->object) || !gtk_html_get_magic_smileys (e->widget))
 		return;
 
 	pos = e->cursor->offset - 1;
