@@ -24,6 +24,7 @@
 #include <config.h>
 #include <string.h>
 #include <stdlib.h>
+#include "gtkhtml-compat.h"
 
 #include <glib.h>
 #include "htmlentity.h"
@@ -316,12 +317,12 @@ html_entity_parse (const gchar *s, guint len)
 		gint i;
 
 		ehash = g_hash_table_new (g_str_hash, html_g_str_case_equal);
-		g_hash_table_freeze (ehash);
+		/* RM2 g_hash_table_freeze (ehash); */
 
 		for (i = 0; i < sizeof (entity_table) / sizeof (entity_table[0]); i++)
 			g_hash_table_insert (ehash, (gpointer) entity_table[i].str, GINT_TO_POINTER (entity_table[i].value));
 
-		g_hash_table_thaw (ehash);
+		/* RM2 g_hash_table_thaw (ehash); */
 	}
 
 	if (len > 0) {
