@@ -73,6 +73,8 @@ typedef enum {
 	HTMLALinkColor,
 	HTMLHighlightColor,
 	HTMLHighlightTextColor,
+	HTMLHighlightNFColor,
+	HTMLHighlightTextNFColor,
 	HTMLSpellErrorColor,
 
 	HTMLColors
@@ -124,13 +126,16 @@ typedef enum {
 	HTML_LIST_TYPE_ORDERED_LOWER_ROMAN,
 	HTML_LIST_TYPE_ORDERED_UPPER_ROMAN,
 	HTML_LIST_TYPE_MENU,
-	HTML_LIST_TYPE_DIR
+	HTML_LIST_TYPE_DIR,
+	HTML_LIST_TYPE_BLOCKQUOTE,
+	HTML_LIST_TYPE_BLOCKQUOTE_CITE,
+	HTML_LIST_TYPE_GLOSSARY_DL,
+	HTML_LIST_TYPE_GLOSSARY_DD
 } HTMLListType;
 
 typedef enum {
 	HTML_OBJECT_FLAG_NONE = 0,
 	HTML_OBJECT_FLAG_SEPARATOR = 1 << 0,
-	HTML_OBJECT_FLAG_NEWLINE = 1 << 1,
 	HTML_OBJECT_FLAG_SELECTED = 1 << 2,
 	HTML_OBJECT_FLAG_ALLSELECTED = 1 << 3,
 	HTML_OBJECT_FLAG_FIXEDWIDTH = 1 << 4,
@@ -170,7 +175,6 @@ typedef enum {
 	HTML_TYPE_IMAGE,
 	HTML_TYPE_IMAGEINPUT,
 	HTML_TYPE_LINKTEXT,
-	HTML_TYPE_MAP,
 	HTML_TYPE_OBJECT,
 	HTML_TYPE_RADIO,
 	HTML_TYPE_RULE,
