@@ -23,6 +23,8 @@
 
 #include <config.h>
 #include <string.h>
+#include <glib/gdataset.h>
+#include <glib/gquark.h>
 
 #include "htmlclue.h"
 #include "htmlclueflow.h"
@@ -50,9 +52,6 @@ HTMLObjectClass html_object_class;
 
 #define HO_CLASS(x) HTML_OBJECT_CLASS (HTML_OBJECT (x)->klass)
 
-
-/* HTMLObject virtual methods.  */
-
 static void
 destroy (HTMLObject *self)
 {
@@ -474,7 +473,7 @@ get_cursor_base (HTMLObject *self,
 static guint
 get_length (HTMLObject *self)
 {
-	return 1;
+	return html_object_accepts_cursor (self) ? 1 : 0;
 }
 
 static guint
@@ -499,7 +498,7 @@ select_range (HTMLObject *self,
 	gboolean selected;
 	gboolean changed;
 
-	selected = length > 0 || (length == -1 && start < html_object_get_length (self)) ? TRUE : FALSE;
+	selected = length > 0 || (length == -1 && start < html_object_get_length (self)) || html_object_is_container (self) ? TRUE : FALSE;
 	changed  = (! selected && self->selected) || (selected && ! self->selected) ? TRUE : FALSE;
 
 	self->selected = selected;
@@ -781,6 +780,18 @@ html_object_set_parent (HTMLObject *o, HTMLObject *parent)
 	o->parent = parent;
 }
 
+
+static void
+frame_offset (HTMLObject *o, 
+	      gint *x_return, gint *y_return)
+{
+	if (html_object_is_frame (o)) {
+		HTMLEngine *e = html_object_get_engine (o, NULL);
+		*x_return -= e->x_offset;
+		*y_return -= e->y_offset;
+	} 
+}
+
 void
 html_object_calc_abs_position (HTMLObject *o,
 			       gint *x_return, gint *y_return)
@@ -791,33 +802,43 @@ html_object_calc_abs_position (HTMLObject *o,
 
 	*x_return = o->x;
 	*y_return = o->y;
+	
+	frame_offset (o, x_return, y_return);
 
 	for (p = o->parent; p != NULL; p = p->parent) {
 		*x_return += p->x;
 		*y_return += p->y - p->ascent;
-		if (html_object_is_frame (p)) {
-			*x_return += GTK_HTML (HTML_FRAME (p)->html)->engine->leftBorder;
-			*y_return += GTK_HTML (HTML_FRAME (p)->html)->engine->topBorder;
-		}
+		
+		frame_offset (p, x_return, y_return);
 	}
 }
 
-void
-html_object_calc_intersection (HTMLObject *o, ArtIRect *intersection, gint x, gint y, gint width, gint height)
+GdkRectangle *
+html_object_get_bounds (HTMLObject *o, GdkRectangle *bounds)
 {
-	ArtIRect clip, area;
-	
-	area.x0 = o->x;
-	area.x1 = o->x + o->width;
-	area.y0 = o->y - o->ascent;
-	area.y1 = o->y + o->descent;
+	if (!bounds)
+		bounds = g_new (GdkRectangle, 1);
 
-	clip.x0 = x;
-	clip.x1 = x + width;
-	clip.y0 = y;
-	clip.y1 = y + height;
+	bounds->x = o->x;
+	bounds->y = o->y - o->ascent;
+	bounds->width = o->width;
+	bounds->height = o->ascent + o->descent;
 
-	art_irect_intersect (intersection, &clip, &area);
+	return bounds;
+}
+
+gboolean
+html_object_intersect (HTMLObject *o, GdkRectangle *result, gint x, gint y, gint width, gint height)
+{
+	GdkRectangle b;
+	GdkRectangle a;
+
+	a.x = x;
+	a.y = y;
+	a.width = width;
+	a.height = height;
+
+	return gdk_rectangle_intersect (html_object_get_bounds (o, &b), &a, result);
 }
 
 
@@ -1392,7 +1413,7 @@ html_object_cursor_backward (HTMLObject *self, HTMLCursor *cursor)
 		return FALSE;
 
 	if (cursor->offset > 1 || (cursor->offset > 0 && (! (prev = html_object_prev_not_slave (self))
-							  || HTML_IS_CLUEALIGNED (prev)))) {
+							  || HTML_IS_CLUEALIGNED (prev) || !html_object_accepts_cursor (prev)))) {
 		cursor->offset --;
 		cursor->position --;
 		return TRUE;
@@ -1440,7 +1461,7 @@ move_object (HTMLObject *obj, HTMLObject * (*next_fn ) (HTMLObject *, HTMLObject
 	return obj;
 }
 
-inline HTMLObject *
+HTMLObject *
 html_object_next_leaf (HTMLObject *self)
 {
 	return move_object (self, html_object_next, html_object_head);
@@ -1455,7 +1476,7 @@ html_object_next_leaf_not_type (HTMLObject *self, HTMLType t)
 	return rv;
 }
 
-inline HTMLObject *
+HTMLObject *
 html_object_prev_leaf (HTMLObject *self)
 {
 	return move_object (self, html_object_prev, html_object_tail);
@@ -1491,11 +1512,22 @@ next_object_uptree_cursor (HTMLObject *obj, HTMLObject * (*next_fn ) (HTMLObject
 /* go down in tree to leaf in way given by down_fn children */
 
 static HTMLObject *
-move_object_downtree_cursor (HTMLObject *obj, HTMLObject * (*down_fn ) (HTMLObject *))
+move_object_downtree_cursor (HTMLObject *obj, HTMLObject * (*down_fn ) (HTMLObject *), HTMLObject * (*next_fn ) (HTMLObject *))
 {
+	HTMLObject *last_obj = obj;
+
 	while ((obj = (*down_fn) (obj))) {
 		if (html_object_accepts_cursor (obj))
 			break;
+		last_obj = obj;
+	}
+
+	if (!obj && last_obj) {
+		obj = last_obj;
+
+		while ((obj = (*next_fn) (obj)))
+			if (html_object_accepts_cursor (obj))
+				break;
 	}
 
 	return obj;
@@ -1511,7 +1543,7 @@ move_object_cursor (HTMLObject *obj, gint *offset, gboolean forward,
 		gboolean found = FALSE;
 		if (((*offset == 0 && forward) || (*offset && !forward)) && html_object_is_container (obj))
 			if ((down = (*down_fn) (obj))) {
-				down = move_object_downtree_cursor (down, down_fn);
+				down = move_object_downtree_cursor (down, down_fn, next_fn);
 				if (down) {
 					if (html_object_is_container (down))
 						*offset = forward ? 0 : 1;
@@ -1531,7 +1563,7 @@ move_object_cursor (HTMLObject *obj, gint *offset, gboolean forward,
 					found = TRUE;
 				} else {
 					HTMLObject *down;
-					down = move_object_downtree_cursor (obj, down_fn);
+					down = move_object_downtree_cursor (obj, down_fn, next_fn);
 					if (down) {
 						if (html_object_is_container (down))
 							*offset = forward ? 0 : 1;
@@ -1546,13 +1578,13 @@ move_object_cursor (HTMLObject *obj, gint *offset, gboolean forward,
 	return obj;
 }
 
-inline HTMLObject *
+HTMLObject *
 html_object_next_cursor (HTMLObject *self, gint *offset)
 {
 	return move_object_cursor (self, offset, TRUE, html_object_next_not_slave, html_object_head);
 }
 
-inline HTMLObject *
+HTMLObject *
 html_object_prev_cursor (HTMLObject *self, gint *offset)
 {
 	return move_object_cursor (self, offset, FALSE, html_object_prev_not_slave, html_object_tail_not_slave);
@@ -1578,6 +1610,12 @@ html_object_set_data (HTMLObject *object, const gchar *key, const gchar *value)
 	g_datalist_set_data_full (&object->object_data, key, g_strdup (value), g_free);
 }
 
+void
+html_object_set_data_full (HTMLObject *object, const gchar *key, const gpointer value, GDestroyNotify func)
+{
+	g_datalist_set_data_full (&object->object_data, key, value, func);
+}
+
 gpointer
 html_object_get_data (HTMLObject *object, const gchar *key)
 {
@@ -1589,10 +1627,9 @@ copy_data (GQuark key_id, gpointer data, gpointer user_data)
 {
 	HTMLObject *o = HTML_OBJECT (user_data);
 
-	/* printf ("copy key: '%s'\n", g_quark_to_string (key_id)); */
-	g_datalist_set_data_full (&o->object_data,
-				  g_strdup (g_quark_to_string (key_id)),
-				  g_strdup ((gchar *) data), g_free);
+	g_datalist_id_set_data_full (&o->object_data,
+				     key_id,
+				     g_strdup ((gchar *) data), g_free);
 }
 
 void
@@ -1611,7 +1648,7 @@ object_save_data (GQuark key_id, gpointer data, gpointer user_data)
 	/* printf ("object %s %s\n", g_quark_to_string (key_id), str); */
 	if (!str) {
 		/* printf ("save %s %s -> %s\n", state->save_data_class_name, g_quark_to_string (key_id), (gchar *) data); */
-		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">",
+		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">-->",
 						state->save_data_class_name, g_quark_to_string (key_id), data);
 		html_engine_set_class_data (state->engine, state->save_data_class_name, g_quark_to_string (key_id), data);
 	}
@@ -1627,13 +1664,12 @@ handle_object_data (gpointer key, gpointer value, gpointer data)
 	/* printf ("handle: %s %s %s %s\n", state->save_data_class_name, key, value, str); */
 	if (!str) {
 		/* printf ("clear\n"); */
-		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" clear=\"%s\">",
+		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" clear=\"%s\">-->",
 						state->save_data_class_name, key);
-		
 		state->data_to_remove = g_slist_prepend (state->data_to_remove, key);
 	} else if (strcmp (value, str)) {
 		/* printf ("change\n"); */
-		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">",
+		html_engine_save_output_string (state, "<!--+GtkHTML:<DATA class=\"%s\" key=\"%s\" value=\"%s\">-->",
 						state->save_data_class_name, key, str);
 		html_engine_set_class_data (state->engine, state->save_data_class_name, key, value);
 	}
@@ -1916,9 +1952,8 @@ html_object_engine_translation (HTMLObject *o, HTMLEngine *e, gint *tx, gint *ty
 		*ty += p->y - p->ascent;
 	}
 
-	*tx = *tx + e->leftBorder - e->x_offset;
-	*ty = *ty + e->topBorder - e->y_offset;
-
+	/* *tx += e->leftBorder; */
+	/* *ty += e->topBorder; */
 }
 
 gboolean
@@ -1957,6 +1992,25 @@ html_object_add_to_changed (GList **changed_objs, HTMLObject *o)
 	*changed_objs = g_list_prepend (*changed_objs, o);
 }
 
+gint
+html_object_get_n_children (HTMLObject *self)
+{
+	return HO_CLASS (self)->get_n_children ? (* HO_CLASS (self)->get_n_children) (self) : 0;
+}
+
+HTMLObject *
+html_object_get_child (HTMLObject *self, gint index)
+{
+	return HO_CLASS (self)->get_child ? (* HO_CLASS (self)->get_child) (self, index) : NULL;
+}
+
+
+gint
+html_object_get_child_index (HTMLObject *self, HTMLObject *child)
+{
+	return HO_CLASS (self)->get_child_index ? (* HO_CLASS (self)->get_child_index) (self, child) : -1;
+}
+
 HTMLClearType
 html_object_get_clear (HTMLObject *self)
 {
