@@ -4,6 +4,7 @@
    Copyright (C) 1997 Martin Jones (mjones@kde.org)
    Copyright (C) 1997 Torben Weis (weis@kde.org)
    Copyright (C) 1999, 2000 Helix Code, Inc.
+   Copyright (C) 2000, 2001, 2002, 2003 Ximian, Inc.
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -29,6 +30,7 @@
 #include "htmlcolor.h"
 #include "htmlcolorset.h"
 #include "htmlentity.h"
+#include "htmltext.h"
 #include "htmlpainter.h"
 
 
@@ -51,6 +53,7 @@ finalize (GObject *object)
 	html_font_manager_finalize (&painter->font_manager);
 
 	html_colorset_destroy (painter->color_set);
+	g_free (painter->font_face);
 
 	/* FIXME ownership of the color set?  */
 
@@ -286,7 +289,7 @@ html_painter_get_font (HTMLPainter *painter, HTMLFontFace *face, GtkHTMLFontStyl
 void
 html_painter_calc_text_size (HTMLPainter *painter,
 			     const gchar *text,
-			     guint len, GList *items, GList *glyphs, gint *line_offset,
+			     guint len, GList *items, GList *glyphs, gint start_byte_offset, gint *line_offset,
 			     GtkHTMLFontStyle font_style,
 			     HTMLFontFace *face,
 			     gint *width, gint *asc, gint *dsc)
@@ -296,67 +299,18 @@ html_painter_calc_text_size (HTMLPainter *painter,
 	g_return_if_fail (text != NULL);
 	g_return_if_fail (font_style != GTK_HTML_FONT_STYLE_DEFAULT);
 
-	(* HP_CLASS (painter)->calc_text_size) (painter, text, len, items, glyphs, font_style, face, width, asc, dsc);
-}
+	(* HP_CLASS (painter)->calc_text_size) (painter, text, len, items, glyphs, start_byte_offset, font_style, face, width, asc, dsc);
 
-static gint
-correct_width (const gchar *text, guint bytes_len, gint *lo, HTMLFont *font)
-{
-	gint delta = 0;
-	gunichar uc;
-	const gchar *s, *end = text + bytes_len;
-	gint skip, line_offset = *lo;
-	gboolean tabs = *lo != -1;
-
-	if (!tabs) {
-		if (font->space_width == font->nbsp_width) {
-			if (font->space_width == font->tab_width) {
-				return 0;
-			} else {
-				while (text < end) {
-					if (*text == '\t')
-						delta += font->space_width - font->tab_width;
-					text ++;
-				}
-
-				return delta;
-			}
-		}
+	if (line_offset) {
+		gint tabs;
+		*width += (html_text_text_line_length (text, line_offset, len, &tabs) - len + tabs)*html_painter_get_space_width (painter, font_style, face);
 	}
-
-	s = text;
-	while (s < end && (uc = g_utf8_get_char (s))) {
-		switch (uc) {
-		case ENTITY_NBSP:
-			line_offset ++;
-			delta += font->space_width - font->nbsp_width;
-			break;
-		case '\t':
-			if (tabs) {
-				skip = 8 - (line_offset % 8);
-				line_offset += skip;
-				delta += skip * font->space_width - font->tab_width;
-			} else {
-				delta += font->space_width - font->tab_width;
-				line_offset ++;
-			}
-			break;
-		default:
-			line_offset ++;
-		}
-		s = g_utf8_next_char (s);
-	}
-
-	if (tabs)
-		*lo = line_offset;
-
-	return delta;
 }
 
 void
 html_painter_calc_text_size_bytes (HTMLPainter *painter,
 				    const gchar *text,
-				    guint bytes_len, GList *items, GList *glyphs, gint *line_offset,
+				    guint bytes_len, GList *items, GList *glyphs, gint start_byte_offset, gint *line_offset,
 				    HTMLFont *font, GtkHTMLFontStyle style,
 				    gint *width, gint *asc, gint *dsc)
 {
@@ -365,8 +319,11 @@ html_painter_calc_text_size_bytes (HTMLPainter *painter,
 	g_return_if_fail (text != NULL);
 	g_return_if_fail (style != GTK_HTML_FONT_STYLE_DEFAULT);
 
-	(* HP_CLASS (painter)->calc_text_size_bytes) (painter, text, bytes_len, items, glyphs, font, style, width, asc, dsc);
-	*width += correct_width (text, bytes_len, line_offset, font);
+	(* HP_CLASS (painter)->calc_text_size_bytes) (painter, text, bytes_len, items, glyphs, start_byte_offset, font, style, width, asc, dsc);
+	if (line_offset) {
+		gint tabs, len = g_utf8_pointer_to_offset (text, text + bytes_len);
+		*width += (html_text_text_line_length (text, line_offset, len, &tabs) - len + tabs)*font->space_width;
+	}
 }
 
 /* The actual paint operations.  */
@@ -426,17 +383,72 @@ html_replace_tabs (const gchar *text, gchar *translated, guint bytes)
 	} while (tab);
 }
 
-gint
-html_painter_draw_text (HTMLPainter *painter,
-			gint x, gint y,
-			const gchar *text, gint len, GList *items, GList *glyphs, gint line_offset)
+static inline GList *
+shift_items (GList *items, gint byte_offset)
 {
+	if (items) {
+		PangoItem *item;
+
+		while (items && (item = (PangoItem *) items->data) && item->offset + item->length <= byte_offset)
+			items = items->next;
+	}
+
+	return items;
+}
+
+static inline GList *
+shift_glyphs (GList *glyphs, gint len)
+{
+	if (glyphs) {
+		PangoGlyphString *str;
+
+		while (glyphs && (str = (PangoGlyphString *) glyphs->data) && len > 0) {
+			len -= str->num_glyphs;
+			glyphs = glyphs->next;
+		}
+	}
+
+	return glyphs;
+}
+
+gint
+html_painter_draw_text (HTMLPainter *painter, gint x, gint y,
+			const gchar *text, gint len, GList *items, GList *glyphs, gint start_byte_offset, gint line_offset)
+{
+	const gchar *tab, *c_text = text;
+	gint bytes, byte_offset = 0;
+
 	g_return_val_if_fail (painter != NULL, line_offset);
 	g_return_val_if_fail (HTML_IS_PAINTER (painter), line_offset);
 
-	(* HP_CLASS (painter)->draw_text) (painter, x, y, text, len, items, glyphs);
+	if (items)
+		items = shift_items (items, start_byte_offset);
 
-	return line_offset;
+	bytes = g_utf8_offset_to_pointer (text, len) - text;
+	while ((tab = memchr (c_text, (unsigned char) '\t', bytes))) {
+		gint c_bytes = tab - c_text;
+		gint c_len = g_utf8_pointer_to_offset (c_text, tab);
+		
+		if (c_bytes)
+			x += (* HP_CLASS (painter)->draw_text) (painter, x, y, c_text, c_len, items, glyphs, start_byte_offset + (c_text - text));
+		if (line_offset == -1)
+			x += html_painter_get_space_width (painter, painter->font_style, painter->font_face);
+		else {
+			line_offset += c_len;
+			x += html_painter_get_space_width (painter, painter->font_style, painter->font_face)*(8 - (line_offset % 8));
+			line_offset += 8 - (line_offset % 8);
+		}
+		c_text += c_bytes + 1;
+		bytes -= c_bytes + 1;
+		byte_offset += c_bytes + 1;
+		items = shift_items (items, start_byte_offset + byte_offset);
+		glyphs = shift_glyphs (glyphs, c_len);
+		len -= c_len + 1;
+	}
+
+	(* HP_CLASS (painter)->draw_text) (painter, x, y, c_text, len, items, glyphs, start_byte_offset + (c_text - text));
+
+	return line_offset + len;
 }
 
 void
@@ -570,9 +582,9 @@ gint
 html_painter_draw_spell_error (HTMLPainter *painter,
 			       gint x, gint y,
 			       const gchar *text,
-			       gint len, GList *items, GList *glyphs)
+			       gint len, GList *items, GList *glyphs, gint start_byte_offset)
 {
-	return (* HP_CLASS (painter)->draw_spell_error) (painter, x, y, text, len, items, glyphs);
+	return (* HP_CLASS (painter)->draw_spell_error) (painter, x, y, text, len, items, glyphs, start_byte_offset);
 }
 
 HTMLFont *
