@@ -25,57 +25,56 @@
 #define _HTMLPAINTER_H_
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gtk/gtkobject.h>
+#include <gdk/gdktypes.h>
+#include <glib-object.h>
 
 #include "gtkhtml-enums.h"
 #include "htmltypes.h"
+#include "htmlfontmanager.h"
 
+
 #define HTML_TYPE_PAINTER                 (html_painter_get_type ())
-#define HTML_PAINTER(obj)                 (GTK_CHECK_CAST ((obj), HTML_TYPE_PAINTER, HTMLPainter))
-#define HTML_PAINTER_CLASS(klass)         (GTK_CHECK_CLASS_CAST ((klass), HTML_TYPE_PAINTER, HTMLPainterClass))
-#define HTML_IS_PAINTER(obj)              (GTK_CHECK_TYPE ((obj), HTML_TYPE_PAINTER))
-#define HTML_IS_PAINTER_CLASS(klass)      (GTK_CHECK_CLASS_TYPE ((klass), HTML_TYPE_PAINTER))
+#define HTML_PAINTER(obj)                 (G_TYPE_CHECK_INSTANCE_CAST ((obj), HTML_TYPE_PAINTER, HTMLPainter))
+#define HTML_PAINTER_CLASS(klass)         (G_TYPE_CHECK_CLASS_CAST ((klass), HTML_TYPE_PAINTER, HTMLPainterClass))
+#define HTML_IS_PAINTER(obj)              (G_TYPE_CHECK_INSTANCE_TYPE ((obj), HTML_TYPE_PAINTER))
+#define HTML_IS_PAINTER_CLASS(klass)      (G_TYPE_CHECK_CLASS_TYPE ((klass), HTML_TYPE_PAINTER))
 
+
 struct _HTMLPainter {
-	GtkObject base;
+	GObject base;
 
+	HTMLFontManager     font_manager;
 	HTMLColorSet       *color_set;
 	HTMLFontFace       *font_face;
 	GtkHTMLFontStyle    font_style;
 
 	gboolean focus;
-	gdouble  magnification;
-	HTMLFontManager *mag_fm;
 };
 
 struct _HTMLPainterClass {
-	GtkObjectClass   base;
+	GObjectClass   base;
 
 	void (* begin)            (HTMLPainter *painter, int x1, int y1, int x2, int y2);
 	void (* end)              (HTMLPainter *painter);
 
-	/* static virtual methods */
-        HTMLFont * (* alloc_font) (gchar *face_name, gdouble size, gboolean points, GtkHTMLFontStyle  style);
-	void       (* ref_font)   (HTMLFont *font);
-	void       (* unref_font) (HTMLFont *font);
-	/* end of static methods */
+        HTMLFont * (* alloc_font) (HTMLPainter *p, gchar *face_name, gdouble size, gboolean points, GtkHTMLFontStyle  style);
+	void       (* ref_font)   (HTMLPainter *p, HTMLFont *font);
+	void       (* unref_font) (HTMLPainter *p, HTMLFont *font);
 
 	void (* alloc_color)      (HTMLPainter *painter, GdkColor *color);
 	void (* free_color)       (HTMLPainter *painter, GdkColor *color);
 
-	guint (* calc_ascent)     (HTMLPainter *p, GtkHTMLFontStyle f, HTMLFontFace *face);
-	guint (* calc_descent)    (HTMLPainter *p, GtkHTMLFontStyle f, HTMLFontFace *face);
-	guint (* calc_text_width) (HTMLPainter *p, const gchar *text, guint len,
-				   GtkHTMLFontStyle font_style, HTMLFontFace *face);
-	guint (* calc_text_width_bytes) (HTMLPainter *p, const gchar *text, guint len,
-					 HTMLFont *font, GtkHTMLFontStyle font_style);
+	void (* calc_text_size) (HTMLPainter *p, const gchar *text, guint len, GList *items, GList *glyphs, gint start_byte_offset,
+				 GtkHTMLFontStyle font_style, HTMLFontFace *face, gint *width, gint *asc, gint *dsc);
+	void (* calc_text_size_bytes) (HTMLPainter *p, const gchar *text, guint len, GList *items, GList *glyphs, gint start_byte_offset,
+				       HTMLFont *font, GtkHTMLFontStyle font_style, gint *width, gint *asc, gint *dsc);
 
 	void (* set_pen)          (HTMLPainter *painter, const GdkColor *color);
 	const GdkColor * (* get_black) (const HTMLPainter *painter);
 	void (* draw_line)        (HTMLPainter *painter, gint x1, gint y1, gint x2, gint y2);
 	void (* draw_rect)        (HTMLPainter *painter, gint x, gint y, gint width, gint height);
-	void (* draw_text)        (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len);
-	gint (* draw_spell_error) (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len);
+	gint (* draw_text)        (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, GList *items, GList *glyphs, gint start_byte_offset);
+	gint (* draw_spell_error) (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, GList *items, GList *glyphs, gint start_byte_offset);
 	void (* fill_rect)        (HTMLPainter *painter, gint x, gint y, gint width, gint height);
 	void (* draw_pixmap)      (HTMLPainter *painter, GdkPixbuf *pixbuf, 
 				   gint x, gint y,
@@ -98,13 +97,11 @@ struct _HTMLPainterClass {
 
 	guint (*get_page_width)     (HTMLPainter *painter, HTMLEngine *e);
 	guint (*get_page_height)    (HTMLPainter *painter, HTMLEngine *e);
-
-	HTMLFontManagerId (*get_font_manager_id) ();
 };
 
 
 /* Creation.  */
-GtkType           html_painter_get_type                                (void);
+GType             html_painter_get_type                                (void);
 HTMLPainter      *html_painter_new                                     (void);
 
 /* Functions to drive the painting process.  */
@@ -139,30 +136,33 @@ void              html_painter_set_font_style                          (HTMLPain
 GtkHTMLFontStyle  html_painter_get_font_style                          (HTMLPainter       *p);
 void              html_painter_set_font_face                           (HTMLPainter       *p,
 									HTMLFontFace      *f);
-HTMLFont         *html_painter_get_html_font                           (HTMLPainter       *painter,
-									HTMLFontFace      *face,
-									GtkHTMLFontStyle   style);
 gpointer          html_painter_get_font                                (HTMLPainter       *painter,
 									HTMLFontFace      *face,
 									GtkHTMLFontStyle   style);
-guint             html_painter_calc_ascent                             (HTMLPainter       *p,
-									GtkHTMLFontStyle   f,
-									HTMLFontFace      *face);
-guint             html_painter_calc_descent                            (HTMLPainter       *p,
-									GtkHTMLFontStyle   f,
-									HTMLFontFace      *face);
-guint             html_painter_calc_text_width                         (HTMLPainter       *p,
+void              html_painter_calc_text_size                          (HTMLPainter       *p,
 									const gchar       *text,
 									guint              len,
+									GList             *items,
+									GList             *glyphs,
+									gint               start_byte_offset,
 									gint              *line_offset,
 									GtkHTMLFontStyle   font_style,
-									HTMLFontFace      *face);
-guint             html_painter_calc_text_width_bytes                   (HTMLPainter       *p,
+									HTMLFontFace      *face,
+									gint              *width,
+									gint              *asc,
+									gint              *dsc);
+void              html_painter_calc_text_size_bytes                    (HTMLPainter       *p,
 									const gchar       *text,
 									guint              len,
+									GList             *items,
+									GList             *glyphs,
+									gint               start_byte_offset,
 									gint              *line_offset,
 									HTMLFont          *font,
-									GtkHTMLFontStyle   font_style);
+									GtkHTMLFontStyle   font_style,
+									gint              *width,
+									gint              *asc,
+									gint              *dsc);
 
 /* The actual paint operations.  */
 void              html_painter_set_pen                                 (HTMLPainter       *painter,
@@ -182,6 +182,9 @@ gint              html_painter_draw_text                               (HTMLPain
 									gint               y,
 									const gchar       *text,
 									gint               len,
+									GList             *items,
+									GList             *glyphs,
+									gint               start_byte_offset,
 									gint               line_offset);
 void              html_painter_fill_rect                               (HTMLPainter       *painter,
 									gint               x,
@@ -238,19 +241,28 @@ gint              html_painter_draw_spell_error                        (HTMLPain
 									gint               x,
 									gint               y,
 									const gchar       *text,
-									gint               len);
-HTMLFont         *html_painter_alloc_font                              (HTMLPainterClass  *pc,
+									gint               len,
+									GList             *items,
+									GList             *glyphs,
+									gint               start_byte_offset);
+HTMLFont         *html_painter_alloc_font                              (HTMLPainter       *painter,
 									gchar             *face_name,
 									gdouble            size,
 									gboolean           points,
 									GtkHTMLFontStyle   style);
-void              html_painter_ref_font                                (HTMLPainterClass  *pc,
+void              html_painter_ref_font                                (HTMLPainter       *painter,
 									HTMLFont          *font);
-void              html_painter_unref_font                              (HTMLPainterClass  *pc,
+void              html_painter_unref_font                              (HTMLPainter       *painter,
 									HTMLFont          *font);
 guint             html_painter_get_space_width                         (HTMLPainter       *painter,
 									GtkHTMLFontStyle   font_style,
 									HTMLFontFace      *face);
+guint             html_painter_get_block_indent_width                  (HTMLPainter       *painter,
+									GtkHTMLFontStyle   font_style,
+									HTMLFontFace      *face);
+guint             html_painter_get_block_cite_width                    (HTMLPainter       *painter,
+									GtkHTMLFontStyle   font_style,
+									HTMLFontFace      *face);
 void              html_painter_draw_embedded                           (HTMLPainter       *painter,
 									HTMLEmbedded      *element,
 									gint               x,
@@ -261,9 +273,12 @@ guint             html_painter_get_page_height                         (HTMLPain
 								        HTMLEngine        *e);
 void              html_painter_set_focus                               (HTMLPainter       *painter,
 									gboolean           focus);
-HTMLPainterClass *html_painter_class_from_id                           (HTMLFontManagerId  id);
-HTMLFontManagerId html_painter_get_font_manager_id                     (HTMLPainter       *painter);
-void              html_painter_set_magnification                       (HTMLPainter       *painter,
-									GtkHTML           *html,
-									gdouble            magnification);
+void              html_replace_tabs                                    (const gchar       *text,
+									gchar             *translated,
+									guint              bytes);
+
+#define HTML_BLOCK_INDENT   "        "
+#define HTML_BLOCK_CITE     "> "
+#define HTML_ALLOCA_MAX     2048
+
 #endif /* _HTMLPAINTER_H_ */
