@@ -23,32 +23,43 @@
 #define _HTMLPRINTER_H
 
 #include <libgnomeprint/gnome-print.h>
+#include <libgnomeprint/gnome-print-job.h>
+#include <libgnomeprint/gnome-print-config.h>
 #include "htmlpainter.h"
 
-
 #define HTML_TYPE_PRINTER                 (html_printer_get_type ())
-#define HTML_PRINTER(obj)                 (GTK_CHECK_CAST ((obj), HTML_TYPE_PRINTER, HTMLPrinter))
-#define HTML_PRINTER_CLASS(klass)         (GTK_CHECK_CLASS_CAST ((klass), HTML_TYPE_PRINTER, HTMLPrinterClass))
-#define HTML_IS_PRINTER(obj)              (GTK_CHECK_TYPE ((obj), HTML_TYPE_PRINTER))
-#define HTML_IS_PRINTER_CLASS(klass)      (GTK_CHECK_CLASS_TYPE ((klass), HTML_TYPE_PRINTER))
+#define HTML_PRINTER(obj)                 (G_TYPE_CHECK_INSTANCE_CAST((obj), HTML_TYPE_PRINTER, HTMLPrinter))
+#define HTML_PRINTER_CLASS(klass)         (G_TYPE_CHECK_CLASS_CAST ((klass), HTML_TYPE_PRINTER, HTMLPrinterClass))
+#define HTML_IS_PRINTER(obj)              (G_TYPE_CHECK_INSTANCE_TYPE ((obj), HTML_TYPE_PRINTER))
+#define HTML_IS_PRINTER_CLASS(klass)      (G_TYPE_CHECK_CLASS_TYPE ((klass), HTML_TYPE_PRINTER))
+
+/* The following macros are used to convert between the HTMLEngine coordinate
+   system (which uses integers) to the GnomePrint one (which uses doubles). */
+
+#define SCALE_ENGINE_TO_GNOME_PRINT(x) ((printer->scale * (x)) / 1024.0)
+#define SCALE_GNOME_PRINT_TO_ENGINE(x) ((gint) (((x) * 1024.0 / printer->scale) + 0.5))
+#define SCALE_GNOME_PRINT_FONT_TO_ENGINE(x) ((gint) (((x) * 1024.0) + 0.5))
 
-
 struct _HTMLPrinter {
 	HTMLPainter base;
 
-	GnomePrintContext *print_context;
+	GnomePrintContext *context;
+	GnomePrintJob     *master;
+	GnomePrintConfig  *config;
+	gdouble scale;
 };
 
 struct _HTMLPrinterClass {
 	HTMLPainterClass base;
 };
 
-
 GtkType      html_printer_get_type                    (void);
-HTMLPainter *html_printer_new                         (GnomePrintContext *print_context);
+HTMLPainter *html_printer_new                         (GnomePrintContext *print_context,
+						       GnomePrintJob     *print_master);
 guint        html_printer_get_page_width              (HTMLPrinter       *printer);
 guint        html_printer_get_page_height             (HTMLPrinter       *printer);
-gdouble      html_printer_scale_to_gnome_print        (gint               x);
+gdouble      html_printer_scale_to_gnome_print        (HTMLPrinter       *printer,
+						       gint               x);
 void         html_printer_coordinates_to_gnome_print  (HTMLPrinter       *printer,
 						       gint               engine_x,
 						       gint               engine_y,
