@@ -45,7 +45,7 @@ static HTMLClueVClass *parent_class = NULL;
 static void
 draw_background_helper (HTMLTableCell *cell,
 			HTMLPainter *p,
-			ArtIRect *paint,
+			GdkRectangle *paint,
 			gint tx, gint ty)
 {
 	HTMLObject *o;
@@ -62,29 +62,35 @@ draw_background_helper (HTMLTableCell *cell,
 			cell->bg_allocated = TRUE;
 		}
 		color = &cell->bg;
-	} else if (t && t->bgColor) {
+	}
+#if 0
+	else if (t && t->bgColor) {
 		html_painter_alloc_color (p, t->bgColor);
 		color = t->bgColor;
 	}
+#endif
 
 	if (cell->have_bgPixmap) {
-		if (cell->bgPixmap->pixbuf) {
-			pixbuf = cell->bgPixmap->pixbuf;
+		if (cell->bgPixmap->animation) {
+			pixbuf = gdk_pixbuf_animation_get_static_image (cell->bgPixmap->animation);
 		}
-	} else if (t && t->bgPixmap)
-		pixbuf = t->bgPixmap->pixbuf;
+	}
+#if 0
+	else if (t && t->bgPixmap && t->bgPixmap->animation)
+		pixbuf = gdk_pixbuf_animation_get_static_image (t->bgPixmap->animation);
 	
+#endif
 	/* FIXME this should be moved into the painter interface */
 	if (!HTML_IS_PLAIN_PAINTER (p))
 		html_painter_draw_background (p,
 					      color,
 					      pixbuf,
-					      tx + paint->x0,
-					      ty + paint->y0,
-					      paint->x1 - paint->x0,
-					      paint->y1 - paint->y0,
-					      paint->x0 - o->x,
-					      paint->y0 - (o->y - o->ascent));
+					      tx + paint->x,
+					      ty + paint->y,
+					      paint->width,
+					      paint->height,
+					      paint->x - o->x,
+					      paint->y - (o->y - o->ascent));
 }
 
 
@@ -166,12 +172,11 @@ draw (HTMLObject *o,
       gint tx, gint ty)
 {
 	HTMLTableCell *cell = HTML_TABLE_CELL (o);
-	ArtIRect paint;
+	GdkRectangle paint;
 
 	
-	html_object_calc_intersection (o, &paint, x, y, width, height);
-	if (art_irect_empty (&paint))
-	    return;
+	if (!html_object_intersect (o, &paint, x, y, width, height))
+		return;
 	
 	draw_background_helper (cell, p, &paint, tx, ty);
 
@@ -236,12 +241,11 @@ draw_background (HTMLObject *self,
 		 gint width, gint height,
 		 gint tx, gint ty)
 {
-	ArtIRect paint;
+	GdkRectangle paint;
 	
 	(* HTML_OBJECT_CLASS (parent_class)->draw_background) (self, painter, x, y, width, height, tx, ty);
 
-	html_object_calc_intersection (self, &paint, x, y, width, height);
-	if (art_irect_empty (&paint))
+	if (!html_object_intersect (self, &paint, x, y, width, height))
 	    return;
 
 	draw_background_helper (HTML_TABLE_CELL (self), painter, &paint, tx, ty);
@@ -462,3 +466,14 @@ html_table_cell_get_fixed_width (HTMLTableCell *cell, HTMLPainter *painter)
 {
 	return html_painter_get_pixel_size (painter) * cell->fixed_width;
 }
+
+gboolean
+html_table_cell_is_empty (HTMLTableCell *cell)
+{
+	g_assert (HTML_IS_TABLE_CELL (cell));
+
+	if (HTML_CLUE (cell)->head && HTML_CLUE (cell)->head == HTML_CLUE (cell)->tail
+	    && HTML_IS_CLUEFLOW (HTML_CLUE (cell)->head) && html_clueflow_is_empty (HTML_CLUEFLOW (HTML_CLUE (cell)->head)))
+		return TRUE;
+	return FALSE;
+}
